/*jslint node: true, camelcase:false*/
module.exports = function (grunt) {
  "use strict";
  var outputDir = "../gh-pages";
  var apiKey = "AIzaSyAKoFgC-xva6WIPbLlkMUpWHVzBHkn5QjM";
  var inputDir = "./src";
  var dirname = (function () {
    var split = __dirname.split("/");
    return split[split.length - 1];
  }());
  var githash = grunt.file.read('.git/refs/heads/master');

  // Project configuration.
  grunt.initConfig({
    // make constants accessible to grunt tasks
    outputDir: outputDir,
    dirname: dirname,
    inputDir: inputDir,
    pkg: grunt.file.readJSON("package.json"),

    // custom task config
    casperfiles: ['test/casperjs/*.js'],
    git: {
      working: 'msbarry/gomap',
      remote: grunt.option('dev') ?
        'https://msbarry@github.com/msbarry/gomap-dev.git' :
        'https://msbarry@github.com/msbarry/gomap.git',
      branch: 'gh-pages'
    },
    deploy: grunt.option('dev') ? {
      host: 'msbarry.github.io',
      path: '/gomap-dev'
    } : {
      host: 'onthegomap.com',
      path: '/'
    },
    casper: {
      dev: { local: inputDir },
      stage: { local: outputDir },
      remote: { remote: 'http://<%= deploy.host %><%= deploy.path %>' }
    },
    watch: {
      files: [
        '*',
        'src/*',
        'src/less/**/*',
        'src/js/**/*',
        'src/img/**/*',
        'test/**/*',
        'build/**/*'
      ],
      tasks: ['jshint', 'qunit:devunit']
    },
    qunit: {
      // run tests against src/ folder
      devunit: [ "test/unit.html" ],
      devint: [ "test/integration.html" ],
      // run integration tests against compiled app
      stage: {
        options: {
          timeout: 100000,
          urls: [
            'http://localhost:<%= connect.stage.options.port %>/<%= dirname %>/test/integration.html?PATH_TO_APP=../<%= outputDir %>'
          ]
        }
      }
    },
    // HTTP server for integration test on staged files
    connect: {
      stage: {
        options: {
          port: 8085,
          base: '..'
        }
      }
    },
    jshint: {
      // default options from root .jshintrc
      options: { jshintrc: ".jshintrc" },
      // merge specific .jshintrc's in with the default one for each subdirectory
      src: {
        files: { src: [ 'src/js/**/*.js', '!src/js/polyfill.js' ] },
        options: { jshintrc: "src/.jshintrc" }
      },
      test: {
        files: { src: [ 'test/**/*.js' ] },
        options: { jshintrc: "test/.jshintrc" }
      },
      build: {
        files: { src: [ 'Gruntfile.js', 'build/**/*.js' ] },
        options: { jshintrc: "build/.jshintrc" }
      }
    },
    imagemin: {
      dist: {
        options: {
          optimizationLevel: 1
        },
        files: {
          '<%= outputDir %>/img/': 'src/img/*',
          '<%= outputDir %>/img/touch/': 'src/img/touch/*'
        }
      }
    },
    targethtml: {
      dist: {
        files: {
          '<%= outputDir %>/index.html': 'src/index.html'
        }
      }
    },
    copy: {
      components: {
        files: [{
          expand: true,
          cwd: 'src/',
          dest: '<%= outputDir %>/',
          src: [
            'components/jquery/jquery.min.js',
            'components/d3/d3.min.js'
          ]
        }]
      },
      fonts: {
        files: [{
          expand: true,
          cwd: 'src/',
          dest: '<%= outputDir %>/',
          src: [
            'fonts/*',
            '!**/*dev.svg'
          ]
        }]
      },
      cname: {
        files: grunt.option('dev') ? {
          '<%= outputDir %>/favicon.ico': 'src/favicon.ico'
        } : {
          '<%= outputDir %>/CNAME': 'src/CNAME',
          '<%= outputDir %>/favicon.ico': 'src/favicon.ico'
        }
      },
      hopkintonRelay: {
        files: {
          '<%= outputDir %>/hopkinton-relay/index.html': 'src/hopkinton-relay/index.html'
        }
      }
    },
    less: {
      prod: {
        options: {
          paths: ["src/less"],
          yuicompress: true
        },
        files: {
          "<%= outputDir %>/css/main.css": "src/less/main.less"
        }
      }
    },
    requirejs: {
      almond: {
        options: {
          baseUrl: "src/js/",
          name: '../components/almond/almond',
          paths: {
            text: "../components/requirejs-text/text"
          },
          stubModules: ['text'],
          include: ['main'],
          insertRequire: ['main'],
          out: outputDir + '/js/main.js',
          optimize: 'uglify2',
          uglify2: {
            output: {
              max_line_len: 500
            }
          },
          wrap: true,
          logLevel: 2
        }
      }
    },
    replace: {
      apikey: {
        src: [outputDir + '/index.html'],
        overwrite: true,
        replacements: [{
          from: 'maps.googleapis.com/maps/api/js?',
          to: 'maps.googleapis.com/maps/api/js?key=' + apiKey + '&amp;'
        }, {
          from: 'OTGM.apikey=null',
          to: 'OTGM.apikey="' + apiKey + '"'
        }]
      },
      githash: {
        src: [outputDir + '/index.html'],
        overwrite: true,
        replacements: [{
          from: '%%githash%%',
          to: githash.substr(0, 7)
        }]
      }
    },
    // ensure files staged properly
    exists: {
      stage: [
        "<%= outputDir %>/index.html",
        // CNAME should only be deployed to production gh-pages, not dev
        grunt.option('dev') ? "!<%= outputDir %>/CNAME" : "<%= outputDir %>/CNAME",
        "<%= outputDir %>/favicon.ico",
        "<%= outputDir %>/css/main.css",
        "<%= outputDir %>/js/main.js",
        "<%= outputDir %>/components/jquery/jquery.min.js",
        "<%= outputDir %>/components/d3/d3.min.js",
        "!<%= outputDir %>/fonts/gomap-icons.dev.svg",
        "<%= outputDir %>/fonts/gomap-icons.eot",
        "<%= outputDir %>/fonts/gomap-icons.svg",
        "<%= outputDir %>/fonts/gomap-icons.ttf",
        "<%= outputDir %>/fonts/gomap-icons.woff",
        "<%= outputDir %>/img/texture-noise.png",
        "<%= outputDir %>/img/touch/apple-touch-icon-144x144-precomposed.png",
        "<%= outputDir %>/img/touch/apple-touch-icon-114x114-precomposed.png",
        "<%= outputDir %>/img/touch/apple-touch-icon-72x72-precomposed.png",
        "<%= outputDir %>/img/touch/apple-touch-icon-57x57-precomposed.png",
        "<%= outputDir %>/img/ogimg.png",
        "<%= outputDir %>/hopkinton-relay/index.html"
      ]
    },
    // make custom modernizr build
    modernizr: {
      "devFile" : "src/components/modernizr/modernizr.js",
      "outputFile" : "<%= outputDir %>/js/vendor/modernizr.js",
      // Based on default settings on http://modernizr.com/download/
      "extra" : {
        "shiv" : false,
        "printshiv" : false,
        "load" : false,
        "mq" : false,
        "cssclasses" : true
      },
      // Based on default settings on http://modernizr.com/download/
      "extensibility" : {
        "addtest" : false,
        "prefixed" : false,
        "teststyles" : false,
        "testprops" : false,
        "testallprops" : false,
        "hasevent" : false,
        "prefixes" : false,
        "domprefixes" : false
      },
      // Tests to implicitly include
      "tests" : [
        "geolocation",
        "csstransforms3d",
        "csstransitions",
        "touch",
        "svg"
      ],
      // Extra custom modernizr tests
      "customTests" : [
        "./src/js/modernizr-tests.js"
      ],
      // Uglify source before saving
      "uglify" : true,
      // By default, this task will crawl your project for references to Modernizr tests.
      // Set to false to disable.
      "parseFiles" : false
    }
  });

  // load custom tasks
  grunt.loadTasks('build');

  // load 3rd party tasks
  grunt.loadNpmTasks('grunt-contrib-jshint');
  grunt.loadNpmTasks('grunt-contrib-requirejs');
  grunt.loadNpmTasks('grunt-contrib-qunit');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-contrib-connect');
  grunt.loadNpmTasks('grunt-targethtml');
  grunt.loadNpmTasks('grunt-text-replace');
  grunt.loadNpmTasks('grunt-contrib-imagemin');
  grunt.loadNpmTasks('grunt-contrib-copy');
  grunt.loadNpmTasks("grunt-modernizr");

  grunt.registerTask("check", "quick static validation, unit test, and integration test against unoptimized code", [
    "jshint",
    "qunit:devunit",
    "qunit:devint",
    "casper:dev"
  ]);

  grunt.registerTask("compile", "run requirejs optimizer to compile app to dest dir", [
    "git-pull",
    "imagemin",
    "targethtml",
    "less:prod",
    "copy",
    "requirejs",
    "replace:githash",
    "modernizr",
    "exists:stage"
  ]);

  grunt.registerTask("build", "compile then test app and add API key if successful", [
    "jshint",
    "qunit:devunit",
    "compile",
    "test-staged",
    "replace:apikey"
  ]);

  grunt.registerTask("test-staged", "run tests against compiled app", [
    "connect:stage",
    "qunit:stage",
    "casper:stage"
  ]);

  // push compiled app to github pages, run acceptance tests against it and
  // roll back if they fail
  grunt.registerTask("deploy", "push compiled app to github pages, run acceptance tests against it and roll back if they fail", [
    "build",
    "git-push"
  ]);

  grunt.registerTask("default", ["build"]);
};