// Make an HTTP request against the given host/path and pass the last modified
// date to the callback function
module.exports = function (host, path, callback, grunt) {
  "use strict";
  var http = require('http'),
    url = require('url'),
    log = grunt ? function (msg) { grunt.log.writeln(msg); } : function () {},
    timeout = 5000,
    httpOptions = {
      host: host,
      port: 80,
      path: path
    },
    timeoutEvent,
    req,
    finish = function (arg) {
      req.abort();
      clearTimeout(timeoutEvent);
      callback(arg);
    };
  log("Get last-modified from http://" + host + path);
  req = http.get(httpOptions,  function handle(resp) {
    var redirectUrl, redirectLocation = resp.headers.location;
    if (resp.statusCode > 300 && resp.statusCode < 400 && redirectLocation) {
      redirectUrl = url.parse(redirectLocation);
      httpOptions.path = redirectUrl.path;
      httpOptions.host = redirectUrl.hostname;
      http.get(httpOptions, handle);
      log("  Redirect to " + redirectLocation);
    } else if (resp.statusCode === 200) {
      finish(resp.headers["last-modified"]);
    } else {
      finish();
    }
  });
  timeoutEvent = setTimeout(finish, timeout);
  return function () {
    req.abort();
    clearTimeout(timeoutEvent);
  };
};