// spawn a list of commands in sequence
module.exports = function (grunt, options) {
  "use strict";
  var opt = function (name, ifNotPresent) {
      var result = options[name] || ifNotPresent;
      if (!result && arguments.length === 1) {
        throw new Error("Option " + name + " not specified.");
      }
      return result;
    },
    dir = opt('dir', '.'),
    cmds = opt('cmds', []),
    succ = opt('succ'),
    err = opt('err'),
    startcwd = process.cwd(),
    spawn = require('child_process').spawn,
    next,
    tasks = cmds.map(function (input) {
      var parts = Array.isArray(input) ? input : input.split(/\s+/),
        cmd = parts[0];
      parts.splice(0, 1);
      return function () {
        grunt.log.writeln("$ " + cmd + " " + parts.join(' '));
        var spawned = spawn(cmd, parts);
        spawned.stdout.setEncoding('utf8');
        spawned.stdout.on('data', function (data) {
          grunt.log.write(data.toString());
        });
        spawned.stderr.on('data', function (data) {
          grunt.log.write(data.toString());
        });
        spawned.on('exit', function (code) {
          if (code === 0) {
            next();
          } else {
            process.chdir(startcwd);
            err();
          }
        });
      };
    });
  next = function () {
    var task = tasks[0];
    if (task) {
      tasks.splice(0, 1);
      task();
    } else {
      process.chdir(startcwd);
      succ();
    }
  };
  process.chdir(dir);
  next();
};