define([
    'jquery',
    'control',
    'map',
    'model',
    'route',
    'url',
    'urlshortener',
    'elevation',
    // 'hideaddressbar',
    'log',
    'Modernizr',
    'text!controls.html'
  ],

  function (
    $,
    Controls,
    Map,
    Model,
    Route,
    URL,
    URLShortener,
    Elevation,
    // hideaddressbar,
    LOG,
    Modernizr,
    CONTROL_HTML) {

    "use strict";
    var events = google.maps.event;
    var OTGM = window.OTGM || {};
    var start = OTGM.startdate;

    var initialize = function () {
      var map = Map.attachTo("#map_canvas");
      var model = new Model();
      var route = new Route(map, model);
      var OTGM = window.OTGM || {};
      var iosapp = Modernizr.ioswebapp;
      OTGM.phase = 'renderctrl';
      Controls.build(map, model, route, CONTROL_HTML);

      // load D3 library if the browser can handle it
      if (Modernizr.svg) {
        $.getScript("components/d3/d3.min.js", function () {
          if (window.d3) {
            $("html").removeClass("no-d3");
            Elevation.bind(map, model, route);
          }
        });
      }

      if (!URL.params.context) {
        URLShortener.load();
      }

      OTGM.phase = 'routerestore';
      if (URL.params.r) {
        // shared route - may be in ios app mode
        var bounds = route.restoreMerged(
          URL.params.r,
          !URL.params.d
        );
        if (bounds) {
          map.fitBounds(bounds);
        }
      } else if (iosapp) {
        // iOS app-specific setup
        route.restore();
      }
      OTGM.phase = 'gmapwait';

      var stopProfile = LOG.profile('Load', 'domcontentload -> map idle');
      events.addListenerOnce(map, 'idle', function () {
        OTGM.phase = 'gomapready';
        $("html").addClass('js-ready');
        // when the map is loaded, trigger a gomapready event or listen for
        // requests for gomapready event
        var $document = $(document);
        var notifyReady = function () {
          Controls.postRenderUpdate(route, map);
          $document.trigger('gomapready', [map, model]);
        };
        $document.on('isgomapready', notifyReady);
        notifyReady();
        OTGM.phase = 'running';
        if (start) {
          ga('send', 'timing', 'Load', 'gomapready', new Date().getTime() - start);
        }
        stopProfile();
        // hideaddressbar();
      });

      events.addListener(map, 'idle', function () {
        $(".gm-style").removeClass("gm-style");
      });
    };

    // API
    return {
      initialize: initialize
    };
  });