// maps from google polyline format to a string that does not need to be escaped in a URI
// see: https://developers.google.com/maps/documentation/utilities/polylinealgorithm
define(function () {

  "use strict";
  // characters that do not need to be URI-escaped
  var safe = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_~-";
  // start of encoded route when we fail to convert to "safe" characters
  var invalidMarker = ".";
  // create a mapping between inpu
  var encode = {}, decode = {};
  for (var i = 63; i < 128; i++) {
    encode[String.fromCharCode(i)] = safe[i - 63];
    decode[safe[i - 63]] = String.fromCharCode(i);
  }
  // convert a character from google polyline format to a URI-safe character
  var googleToURISafe = function (character) { return encode[character]; };
  // convert a character from URI-safe to what the google polyline algorithm expects
  var URISafeToGoogle = function (character) { return decode[character]; };
  // map each character in a string using the function provided
  var mapString = function (string, func) {
    var result = [];
    var len = string.length;
    for (var i = 0; i < len; i++) {
      result.push(func(string[i]));
    }
    return result.join("");
  };
  // returns true if any character in the string doesn't have an associated encoding
  var invalid = function (string) {
    for (var i = string.length - 1; i >= 0; i--) {
      if (!encode[string[i]]) { return true; }
    }
    return false;
  };
  // convert an entire string from google to URI-safe, or if it is invalid
  // then prepend invalidMarker
  var compress = function (input) {
    if (input === null) { return input; }
    if (invalid(input)) { return invalidMarker + input; }
    return mapString(input, googleToURISafe);
  };
  // convert an entier string from URI-safe to google polyline format, unless
  // it starts with invalidMarker
  var decompress = function (input) {
    if (input === null) { return input; }
    if (input.indexOf(invalidMarker) === 0) { return input.substr(1); }
    return mapString(input, URISafeToGoogle);
  };

  // API
  return {
    compress: compress,
    decompress: decompress
  };
});