// setup logic to show/hide dropdown when dropdown control is clicked
define([
    'jquery',
    'Modernizr'
  ],

  function (
    $,
    Modernizr) {

    "use strict";

    var TOGGLE_MODAL = '.toggle-modal';

    // wrap a jquery callback in a function that filters out events triggered on
    // contents of any of the elements contained in $all
    var except = function (callback, $all) {
      return function (event) {
        var clicked = event.target;
        var nomatch = true;
        $all.each(function () {
          nomatch = nomatch && this !== clicked && !$.contains(this, clicked);
          return nomatch;
        });

        // only hide when event is NOT occurring on a or b
        if (nomatch) {
          callback();
        }
      };
    };

    return function ($$) {
      var $doc = $(document.documentElement);
      // hook up dropdown behavior
      var $dropdowns = $();
      $$("[data-dropdown]").each(function () {
        var $controller = $(this);
        var dropdownID = $controller.data("dropdown");
        var $dropdown = $$("#" + dropdownID);
        var toggle = function () {
          $dropdown.toggleClass('active');
          $doc.toggleClass(dropdownID + '-active');
          if ($dropdown.hasClass("active")) {
            $dropdown.trigger("show");
          } else {
            $dropdown.trigger("close");
          }
        };
        if (!$dropdown.get(0)) {
          throw new Error("No div found for data-dropdown=" + dropdownID);
        }
        $controller.fastClick(toggle);
        $dropdowns = $dropdowns.add($dropdown);
      });
      var hideId = function (id, $dropdown) {
        id = id === "nav" ? "options-dropdown" : id;
        $dropdown = $("#" + id);
        if ($dropdown.hasClass("active")) {
          $dropdown.trigger("close");
        }
        $dropdown.removeClass('active');
        $doc.removeClass(id + '-active');
      };

      // make each dropdown hide when anything besides dropdown and its
      // controller is touched
      $dropdowns.each(function () {
        var $dropdown = $(this);
        var id = $dropdown.attr("id");
        var hide = $.proxy(hideId, null, id);
        var action = except(hide, $$("[data-dropdown=" + id + "]").add($dropdown));
        $("#inner-wrap").on('touchstart mousedown', action);
        $dropdown.find(".close").fastClick(hide);
        $dropdown.on('fastclick-done', hide);
        $dropdown.on('click', TOGGLE_MODAL, hide);
      });

      // extra customizations for off canvas menu
      var hideOffCanvasMenu = $.proxy(hideId, null, 'options-dropdown');
      var fixOverlapInput = function () {
        var $overlap = $(".geocode, .overlap .map-btn");
        $overlap.blur();
        // prevent ghost-focus
        if (Modernizr.usefastclick) {
          $overlap.attr('disabled', 'true');
          setTimeout(function () {
            $overlap.attr('disabled', null);
          }, 600);
        }
      };
      $("#nav").on('fastclick-done', hideOffCanvasMenu);
      $(document).on('geocode', hideOffCanvasMenu);
      $(document).on('show', '#options-dropdown', function () {
        hideId("show-link");
        $("#nav").scrollTop(0);
        fixOverlapInput();
      });
      $(document).on('close', '#options-dropdown', fixOverlapInput);
    };
  }
);