define([
    'jquery',
    'Modernizr',
    'elevationprovider',
    'elevationdisplay'
  ],

  function (
    $,
    Modernizr,
    ElevationProvider,
    ElevationDisplay) {

    "use strict";

    var MIN_POINTS = 3,
        MAX_POINTS = 150,
        RESOLUTION = 50; // meters

    var getPointsForMeters = function (meters) {
      if (meters < MIN_POINTS * RESOLUTION) {
        return MIN_POINTS;
      } else if (meters > MAX_POINTS * RESOLUTION) {
        return MAX_POINTS;
      } else {
        return Math.ceil(meters / RESOLUTION);
      }
    };

    var bind = function (map, model, route) {
      var $model = $(model),
          $route = $(route),
          lastData,
          lastExtraInfo,
          display = ElevationDisplay.attachTo("#elevation", model, map);

      var requestElevation = function (fromShow) {
        if (model.getElevation()) {
          var path = route.getMergedPath();
          var length = model.getMeters();
          if (lastData) {
            display.data(length, lastData, path, lastExtraInfo);
          } else if (path && path.length > 1) {
            if (fromShow) { display.loading(length); }
            else { display.disableMouseover(); }
            ElevationProvider.getElevation(path, function (data, extraInfo) {
              lastData = data;
              lastExtraInfo = extraInfo;
              display.data(length, data, path, extraInfo);
            }, function () {
              // received error
              display.empty();
            }, getPointsForMeters(length), length);
          } else {
            display.empty();
          }
        }
      };

      $model.on('elevation', function (e, newVal) {
        if (newVal) {
          requestElevation(true);
        } else {
          display.hide();
        }
      });
      $route.on('change', function () {
        lastData = null;
        requestElevation();
      });
      $(window).on('resize', function () {
        if (model.getElevation()) {
          display.resize();
        }
      });
      $model.on('unit', function () {
        if (model.getElevation()) {
          display.unit();
        }
      });
      $route.on('mouseover', function (evt, data) {
        if (!data || !model.getElevation()) {
          display.mouseoutRoute();
        } else if (data) {
          display.mouseoverRoute(data);
        }
      });
      $(document).on('printing', function () {
        if (model.getElevation()) {
          display.resize();
        }
      });

      return display;
    };

    return {
      bind: bind,
      getPointsForMeters: getPointsForMeters,
      MIN: MIN_POINTS,
      MAX: MAX_POINTS,
      RES: RESOLUTION
    };
  }
);