define([
    'jquery',
    'Modernizr'
  ],

  function (
    $,
    Modernizr) {

    "use strict";

    var activeClass = "fastclicking";
    var timeBeforeActiveClass = 75;
    var coordinates = [];
    var haveAddListener = !!document.addEventListener;

    var preventGhostClick = function(x, y) {
      coordinates.push(x, y);
      window.setTimeout(pop, 2500);
    };

    var pop = function() {
      coordinates.splice(0, 2);
    };

    var bustClicks = function (event) {
      for (var i = 0; i < coordinates.length; i += 2) {
        var x = coordinates[i];
        var y = coordinates[i + 1];
        if (Math.abs(event.clientX - x) < 25 && Math.abs(event.clientY - y) < 25) {
          event.stopPropagation();
          event.preventDefault();
        }
      }
    };

    var createFastButton = function ($element, handler) {
      var startX, startY, inProgress;
      var onTouchStart = function (event) {
        inProgress = true;
        var touch = event.originalEvent.touches[0];
        $element.on('touchend', onClick);
        document.addEventListener('touchmove', onTouchMove, true);

        startX = touch.clientX;
        startY = touch.clientY;

        setTimeout(function () {
          if (inProgress) {
            $element.addClass(activeClass);
          }
        }, timeBeforeActiveClass);
      };
      var onTouchMove = function (event) {
        if (Math.abs(event.touches[0].clientX - startX) > 10 ||
            Math.abs(event.touches[0].clientY - startY) > 10) {
          reset();
        }
      };
      var onClick = function (event) {
        event.stopPropagation();
        reset();
        handler.call($element[0], event);
        $element.trigger('fastclick-done');

        if (event.type === 'touchend') {
          preventGhostClick(startX, startY);
          event.preventDefault();
        }
      };
      var reset = function() {
        inProgress = false;
        $element.off('touchend');
        document.removeEventListener('touchmove', onTouchMove);
        $element.removeClass(activeClass);
      };

      $element.on('touchstart', onTouchStart);
      $element.on('click', onClick);
    };

    $.fn.fastClick = function (handler) {
      var $elem = $(this);
      // if don't have addEventListener, just use regular click
      if (!haveAddListener || $elem.hasClass("slowclick") || !Modernizr.usefastclick) {
        // slow-click class forces to just use regular click event
        return $elem.click(function (event) {
          handler.call(this, event);
          $elem.trigger('fastclick-done');
        });
      } else {
        return $elem.each(function () {
          createFastButton($(this), handler);
        });
      }
    };

    // need useCapture flag
    if (haveAddListener) {
      document.addEventListener('click', bustClicks, true);
    }
  }
);