define([
    'jquery',
    'log'
  ],

  function (
    $,
    LOG) {

    "use strict";
    var MIN_MS_BETWEEN_REQUEST = 500;
    var geocoder;
    var GeocoderStatus = google.maps.GeocoderStatus;
    var nextReq = 0;
    var OTGM = window.OTGM || {};

    // wrapper function
    var geocode = function (address, callback, error) {
      geocoder = geocoder || new google.maps.Geocoder();
      var now = new Date().getTime();
      if (now >= nextReq) {
        nextReq = MIN_MS_BETWEEN_REQUEST + now;
        geocoder.geocode({
          address: address
        }, function (result, status) {
          LOG('Geocode', 'Response', status);
          if (status !== GeocoderStatus.OK || !result[0]) {
            // no success, invoke err handler
            error();
          } else if (result[0].geometry && result[0].geometry.viewport) {
            callback(result[0].geometry.viewport);
          } else if (result[0].geometry && result[0].geometry.bounds) {
            callback(result[0].geometry.bounds);
          } else {
            error();
            LOG.error("Geocode Response Malformed", [status, JSON.stringify(result)].join(":"));
          }
        });
      } else {
        // just drop requests made closer than 1/2s apart
      }
    };

    geocode.reset = function () {
      nextReq = 0;
    };
    OTGM.resetGeocode = geocode.reset;

    // API
    return geocode;
  }
);