define(function () {
  "use strict";

  // private utilities
  var LatLng = google.maps.LatLng;
  var sqr = function (x) { return x * x; };
  var distSquared = function (p1, p2) {
    // TODO unweight longitude diff by cosine of latitude
    // not noticing any change, so leave it out for now
    return sqr(p1.lng() - p2.lng()) + sqr(p1.lat() - p2.lat());
  };

  // Representation of a line segment, used to determine distance to a point
  var Line = function(p1, p2) {
    this.p1 = p1;
    this.p2 = p2;
    this.lengthSquared = distSquared(p1, p2);
  };
  Line.prototype = {
    // The weighting function used for RDP, since we are just ordering by
    // weights and don't care about the actual weights themselves, use the
    // distance squared, which involves less CPU work
    getRatio: function (point) {
      var p1 = this.p1, p2 = this.p2;
      var segmentLength = this.lengthSquared;
      // line has no length
      if (segmentLength === 0) {
        return distSquared(point, p1);
      }
      // ratio on segment from p1 to p2 that is closest to point
      return ((point.lng() - p1.lng()) * (p2.lng() - p1.lng()) +
        (point.lat() - p1.lat()) * (p2.lat() - p1.lat())) / segmentLength;
    },
    distanceToSquared: function (point) {
      var p1 = this.p1, p2 = this.p2;
      var t = this.getRatio(point);
      if (t < 0) {
        // point is past p1
        return distSquared(point, p1);
      }
      if (t > 1) {
        // point is past p2
        return distSquared(point, p2);
      }
      // point is between p1 and p2, find the closest point along the line
      // segment and calculate distance from there to point
      return distSquared(point, new LatLng(
        p1.lat() + t * (p2.lat() - p1.lat()),
        p1.lng() + t * (p2.lng() - p1.lng())
      ));
    },
    distanceTo: function (point) {
      return Math.sqrt(this.distanceToSquared(point));
    }
  };

  return Line;
});