define([
    'jquery',
    'store',
    'gps',
    'url',
    'follow',
    'Modernizr',
    'gmapsingleclick',
    'print'
  ],

  function (
      $,
      Store,
      GPS,
      URL,
      follow,
      Modernizr,
      setupSingleClickEvent,
      setupPrint) {

    "use strict";

    // imports
    var maps = google.maps;
    var Map = maps.Map;
    var LatLng = maps.LatLng;
    var events = maps.event;
    var MapTypeId = maps.MapTypeId;

    // use new visual refresh until it becomes the default
    maps.visualRefresh = true;

    // constants
    var BOSTON_MARATHON_START = new LatLng(42.2297, -71.5181);
    var DEFAULT_ZOOM = 12;
    var DEFAULT_MAP_TYPE = MapTypeId.ROADMAP;
    var VALID_MAPTYPES = [
      MapTypeId.ROADMAP,
      MapTypeId.HYBRID,
      MapTypeId.TERRAIN
    ];

    // create the map on a div and return the gmap object itself
    var attachTo = function (selector) {
      var $map = $(selector);
      var lastLat = Store.getFloatInRange('lat', -85, 85);
      var lastLng = Store.getFloatInRange('lng', -180, 180);
      var haveCenter = lastLat !== null && lastLng !== null;
      var zoom = Store.getPositiveInt('zoom', DEFAULT_ZOOM);
      var mapType = Store.getOneOf('maptype', 'm', VALID_MAPTYPES, DEFAULT_MAP_TYPE);
      var start;
      var requestGeo = true;
      var iosapp = Modernizr.ioswebapp;

      // initialize the map from values stored in localStorage
      if (haveCenter) {
        // don't request geolocation on subsequent load when running in iOS webapp mode
        requestGeo = !iosapp;
        start = new LatLng(lastLat, lastLng);
      } else {
        start = BOSTON_MARATHON_START;
      }
      var map = new Map($map[0], {
        center: start,
        zoom: zoom,
        mapTypeId: mapType,
        mapTypeControl: false,
        streetViewControl: false,
        zoomControl: !Modernizr.useoffcanvasmenu
      });
      var follower = follow.attachTo(map, haveCenter);

      // trigger initial "follow" event
      if (!URL.params.context && requestGeo && !GPS.dontDisturb()) {
        follower.toggle(true);
      }

      // store map state persistently after every drag/zoom finishes animating
      events.addListener(map, 'idle', function () {
        var center = map.getCenter();
        Store.set('lat', center.lat());
        Store.set('lng', center.lng());
        Store.set('zoom', map.getZoom());
      });

      // store map type any time it changes
      events.addListener(map, 'maptypeid_changed', function () {
        Store.set('maptype', map.getMapTypeId());
      });

      // add a "singleclick" event to the map that fires when the user clicks
      // once and that doesn't turn into a double click
      setupSingleClickEvent(map);

      // setup listeners to resize the map properly when printing
      setupPrint(map);

      return map;
    };

    // API
    return {
      attachTo: attachTo
    };
  }
);