(function (Modernizr) {
  "use strict";
  var ua = navigator.userAgent || "";
  var ipad = !!ua.match(/iPad/i);
  var iphone = !!ua.match(/iPhone/i);
  var ipod = !!ua.match(/iPod/i);
  var android = !!ua.match(/Android/i);
  var iosdevice = ipad || iphone || ipod;
  var touchDevice = android || iosdevice;
  var chromeApp = window.chrome && window.chrome.app && window.chrome.app.isInstalled;
  var chromeWebstore = window.chrome && window.chrome.webstore;

  // detect if running full-screen iOS webapp launched from the home screen
  Modernizr.addTest('ioswebapp', !!window.navigator.standalone );

  // detect iOS devices
  Modernizr.addTest('ipad', ipad);
  Modernizr.addTest('iphone', iphone);
  Modernizr.addTest('ipod', ipod);

  // Google Maps API V3 supports: 
  // For desktop:
  //   Internet Explorer 8–10 inclusive (Windows)
  //   The current and previous version of Firefox (Windows, Mac OS X, Linux)
  //   The current and previous version of Chrome (Windows, Mac OS X, Linux)
  //   The current and previous version of Safari (Mac OS X)
  // For Android:
  //   Default browser on Android 2.3+ (which is Chrome on 4.1+)
  // For iOS:
  //   Mobile Safari on the current and previous major version of iOS
  //   Current version of Chrome for iOS

  // progressively enhance android and ios with better mobile features, non-mobile
  // will still work fine on different mobile browsers but you won't get fast-click
  // or off canvas menu
  Modernizr.addTest('useoffcanvasmenu', touchDevice);
  Modernizr.addTest('usefastclick', iosdevice);
  Modernizr.addTest('chrome-app', chromeApp);
  Modernizr.addTest('can-install-chrome-app', chromeWebstore && !chromeApp);
}(window.Modernizr));