// Sample a path at periodic intervals.
define(function () {

  "use strict";

  var sherical = google.maps.geometry.spherical;
  var computeLength = sherical.computeLength;
  var computeDistanceBetween = sherical.computeDistanceBetween;
  var interpolate = sherical.interpolate;

  var sample = function (options) {
    var path = options.path;
    var num = options.numPoints;
    var length = computeLength(path);
    var sampleLength = length / (num - 1);
    var next = sampleLength;
    var dist = 0;
    var result = [];
    var last;

    if (path.length > 0) {
      result.push(path[0]);
      if (path.length > 1) {
        path.forEach(function (point) {
          if (last) {
            var len = computeDistanceBetween(last, point);
            while (result.length < num - 1 && next < len + dist) {
              var ratio = (next - dist) / (len);
              result.push(interpolate(last, point, ratio));
              next += sampleLength;
            }
            dist += len;
          }
          last = point;
        });
        result.push(path[path.length - 1]);
      }
    }

    return result;
  };

  return sample;
});