// register a devicemotion listener to determine when a device is being
// shaked, then invoke a global "shake" event
define([
    'jquery'
  ],

  function ($) {

    "use strict";

    // constants
    var DIRECTION_CHANGES_REQUIRED = 3;
    var ACCELERATION_THRESHOLD = 7;
    var MAX_DURATION = 1000;
    var $win = $(window);

    // shake-tracking state
    var dir;
    var changes;

    var reset = function () {
      dir = {x: 0, y: 0, z: 0};
      changes = {x: [], y: [], z: []};
    };

    reset();

    // handle when acceleration of device goes over ACCELERATION_THRESHOLD in a
    // particular direction for the first time
    var switchDir = function (d) {
      var now = new Date().getTime();
      var isShake = false;
      changes[d].push(now);
      if (changes[d].length >= DIRECTION_CHANGES_REQUIRED) {
        var last = changes[d].shift(1);
        isShake = (now - last) < MAX_DURATION;
      }
      return isShake;
    };

    var handleDeviceMotion = function (event) {
      var accel = event.originalEvent.acceleration;
      var check = function (d) {
        var handled = false;
        if (accel && accel[d] < -ACCELERATION_THRESHOLD && dir[d] >= 0) {
          dir[d] = -1;
          handled = switchDir(d);
        } else if (accel && accel[d] > ACCELERATION_THRESHOLD && dir[d] <= 0) {
          dir[d] = 1;
          handled = switchDir(d);
        }
        return handled;
      };
      if (check('x') || check('y') || check('z')) {
        reset();
        $win.trigger('shake');
      }
    };

    $win.on("devicemotion", handleDeviceMotion);

    return {
      handleDeviceMotion: handleDeviceMotion,
      reset: reset
    };
  }
);