// wrap 3rd party store.js library
// back persisted parameters by values from URL when map is embedded
define([
    'jquery',
    'url',
    '../components/store.js/store'
  ],

  function (
    $,
    URL) {

    "use strict";
    /*global store*/

    var api = {};

    var nada = function () {};
    var either = function (regular, fallback) {
      return function () {
        var result;
        try {
          if (URL && URL.params && URL.params.context) {
            result = fallback.apply(this, arguments);
          } else {
            result = regular.apply(this, arguments);
          }
        } catch (e) {
          // error accessing store
        }
        return result;
      };
    };
    // get the value of a key from persistent storage, only if between lo and hi
    var getFloatInRange = function (key, lo, hi) {
      var value = api.get(key);
      var parsed = parseFloat(value, 10);
      if (!$.isNumeric(parsed) || parsed < lo || parsed > hi) {
        parsed = null;
      }
      return parsed;
    };

    // get the value of a key from persistent storage only if it is in a list of
    // valid values - otherwise return defaultValue
    var getOneOf = function (key, alias, values, defaultValue) {
      var value = api.get(alias) || api.get(key);
      return values.indexOf(value) === -1 ? defaultValue : value;
    };

    // get the value of a key from persistent storage only if it is a positive
    // int, otherwise return default value
    var getPositiveInt = function (key, defaultValue) {
      var value = api.get(key);
      var parsed = parseInt(value, 10);
      if (!$.isNumeric(parsed) || parsed < 0) {
        parsed = defaultValue;
      }
      return parsed;
    };

    // return the boolean value of a key
    var getBoolean = function (key) {
      return api.get(key) === 'true';
    };

    var wrap = function (fn) {
      return function () {
        try {
          return fn.apply(this, arguments);
        } catch (e) {
          // error accessing store
        }
      };
    };

    var setup = function () {
      api.get = either(store.get, function (key) { return URL.params[key]; });
      api.set = either(store.set, nada);
      api.clear = either(store.clear, nada);
      api.remove = either(store.remove, nada);
      api.setup = setup;
      api.getPositiveInt = getPositiveInt;
      api.getOneOf = getOneOf;
      api.getFloatInRange = getFloatInRange;
      api.getBoolean = getBoolean;
      api.force = {
        get: wrap(store.get),
        set: wrap(store.set)
      };
    };
    setup();

    return api;
  }
);