define([
    'jquery',
    'log'
  ],

  function (
    $,
    LOG) {

    // global gapi
    "use strict";
    var id = 0;
    var cache = {};
    var $doc = $(document);
    var OTGM = window.OTGM || {};

    // handler for JSONP callback
    var makeRequest = function (funcName, url) {
      var request = gapi.client.urlshortener.url.insert({
        resource: {
          longUrl: url
        }
      });
      request.execute(function (response) {
        if (response.error) {
          $doc.trigger(funcName, response);
          LOG.error("URL Shortener", JSON.stringify(response.error));
        } else {
          $doc.trigger(funcName, response.id);
          LOG("Share Shorten", "URL Shortener", "OK");
        }
      });
    };

    // returned cached response if available or invoke tinyURL API
    var shorten = function (url, callback, error) {
      ready(function () {
        var last = cache[url];
        if (typeof last === "string") {
          // already have this URL cached
          callback(last);
        } else {
          var thisId = typeof last === "number" ? last : ++id;
          var funcName = "url" + thisId;
          // setup listener
          $doc.one(funcName, function (arg, result) {
            if (result.error) {
              delete cache[url];
              if (error) {
                error();
              }
            } else {
              cache[url] = result;
              if (callback) {
                callback(result);
              }
            }
          });
          if (!cache.hasOwnProperty(url)) {
            // no request pending, start one
            cache[url] = id;
            makeRequest(funcName, url);
          }
        }
      });
    };

    var shortenerReady = function (error) {
      if (!error) {
        $doc.trigger('urlshortenerready');
      } else {
        LOG.error("URL shortener gapi.client load", JSON.stringify(error));
      }
    };

    window.OTGMgapiclientload = function () {
      gapi.client.setApiKey(OTGM.apikey);
      gapi.client.load('urlshortener', 'v1', shortenerReady);
    };

    var ready = function (callback) {
      if (callback) {
        if (window.gapi && gapi.client && gapi.client.urlshortener) {
          callback();
        } else {
          $doc.one('urlshortenerready', callback);
        }
      }
    };

    var load = function () {
      cache = {};
      $.getScript("https://apis.google.com/js/client.js?onload=OTGMgapiclientload")
        .fail(function (jqxhr, a, b) {
          LOG.error('Load client.js error', [a, b].join(" "));
        });
    };

    // API
    return {
      shorten: shorten,
      ready: ready,
      load: load
    };
  }
);