define([
    "jquery"
  ],

  function ($) {

    "use strict";

    // round a number to a certain number of decimal points
    var round = function (num, decimals) {
      var raised = Math.pow(10, decimals || 0);
      return Math.round(num * raised) / raised;
    };

    // return true if two numbers are within a threshold of 1e-5
    var numbersAreClose = function (num1, num2) {
      return Math.abs(num1 - num2) <= 1e-5;
    };

    // return true if two points have lats and lngs within a threshold of 1e-5
    var pointsAreClose = function (p1, p2) {
      return numbersAreClose(p1.lat(), p2.lat()) &&
        numbersAreClose(p1.lng(), p2.lng());
    };

    // convert object key->value pairs to value->key and throw exception if any
    // values collide
    var invert = function (obj) {
      var result = {};
      $.each(obj, function (key, value) {
        if (result.hasOwnProperty(value)) {
          throw new Error("Both '" + result[value] + "' and '" + key + "' map to '" + value + "'");
        }
        result[value] = key;
      });
      return result;
    };

    // add or remove a class from a jQuery object based on the value of the 2nd
    // arg
    var select = function ($control, value) {
      if (value) {
        $control.addClass('checked');
      } else {
        $control.removeClass('checked');
      }
    };

    // API
    return {
      round: round,
      pointsAreClose: pointsAreClose,
      numbersAreClose: numbersAreClose,
      invert: invert,
      select: select
    };
  }
);