/* global Modernizr, jQuery */
(function () {
  "use strict";
  var ignoreError = function (msg) {
    return ignoreErrors || msg.indexOf("evaluating 'a[G]'") > -1;
  };
  var casper = require('casper').create({
    timeout: 30000,
    onAlert: function (self, m) {
      if (!ignoreError(m)) {
        self.die('Alert:' + m, 1);
      } else {
        self.test.comment("Ignoring alert: " + m);
      }
    },
    onLoadError: function (self, m) {
      self.die('Resource load error: ' + m, 1);
    },
    clientScripts: [
      // fake out geolocation
      'test/unit/geolocation.mock.js',
      // and start with a fresh localStorage state
      'test/util/clearlocalstorage.js'
    ],
    viewportSize: {
      width: 1024,
      height: 768
    }
  });
  var PHONE_DROPDOWN_CTRL = ".visible-phone .dropdown-control";
  var WIDE_DROPDOWN_CTRL = ".hidden-phone .dropdown-control";
  var DROPDOWN = "#options-dropdown";
  var SHARE_BTN = "button.share";
  var SHARE_LI = "li.share";
  var SHARE = "#show-link";
  var TINYURL = "#short-url";
  var CONTROLS = "#controls";
  var ADDTHIS = ".addthis_toolbox";
  var PHONE_UNDO = ".visible-phone .undo-ctrl";
  var WIDE_UNDO = ".hidden-phone .undo-ctrl";
  var TOP_CLEAR = "button.clear-ctrl";
  var TOP_REVERSE = "button.reverse-ctrl";
  var DROPDOWN_REVERSE = "li.reverse-ctrl";
  var WALK = ".walk";
  var BIKE = ".bike";
  var DRIVE = ".drive";
  var DIRECT = ".direct";
  var DISTANCE = ".distance";
  var BIKE_LAYER = ".bikelayer-ctrl";
  var FEEDBACK_IFRAME = "iframe[name=feedback]";
  var ignoreErrors = false;
  var assertResource = function (name) {
    casper.then(function () {
      this.test.assertTrue(this.resourceExists(name), name + " exists");
    });
  };

  var waitForLoad = function () {
    casper.then(function () {
      this.test.comment("wait for load and timing");
      this.options.clientScripts = [];
      this.evaluate(function () {
        window.gaargs = [];
        window.gaorig = window.ga;
        window.ga = function () {
          window.gaargs.push([].slice.apply(arguments).join(','));
        };
      });
    });

    casper.then(function () {
      this.evaluate(function () {
        window.ready = false;
        jQuery(document).on('gomapready', function (e, map) {
          window.ready = true;
          window.map = map;
        });
        // just in case gomap alrady loaded
        jQuery(document).trigger('isgomapready');
      });
    });

    casper.waitFor(function () {
      return this.evaluate(function () {
        return window.ready && document.styleSheets.length > 1 && jQuery(".addthis_toolbox").is(":visible");
      });
    });

    casper.waitFor(function () {
      var gaargs = this.evaluate(function () { return window.gaargs.join(';'); });
      var ready = /send,timing,Load,domcontentload -> map idle,[0-9]*/.test(gaargs) &&
        /send,timing,Load,gomapready,[0-9]*/.test(gaargs);
      return ready;
    });

    casper.then(function () {
      this.evaluate(function () { window.ga = window.gaorig; });
    });
  };
  var shouldSee = function (selector) {
    casper.test.assertTrue(
      casper.visible(selector),
      selector + " visible");
  };
  var shouldNotSee = function (selector) {
    casper.test.assertFalse(
      casper.visible(selector) ,
      selector + " NOT visible");
  };
  var textShouldBe = function (selector, text) {
    casper.test.assertEquals(
      casper.fetchText(selector),
      text, "text of " + selector);
  };
  var shouldBeSelected = function (id) {
    casper.test.assertTrue(
      casper.evaluate(function (id) {
        return jQuery(id).hasClass('checked');
      }, id), id + " selected");
  };
  var shouldNotBeSelected = function (id) {
    casper.test.assertTrue(
      casper.evaluate(function (id) {
        return !jQuery(id).hasClass('checked');
      }, id), id + " NOT selected");
  };
  var assertModernizrDetected = function (detect) {
    casper.test.assertTrue(
      casper.evaluate(function (detect) {
        return typeof Modernizr[detect] === 'boolean';
      }, detect), detect + " detected by Modernizr"
    );
    casper.test.assertTrue(
      casper.evaluate(function (detect) {
        return jQuery('html').hasClass(detect) || jQuery('html').hasClass('no-' + detect);
      }, detect), detect + " css class added by Modernizr"
    );
  };
  var fakeModernizr = function (feature, value) {
    casper.evaluate(function (feature, value) {
      if (value) {
        jQuery('html').addClass(feature).removeClass('no-' + feature);
      } else {
        jQuery('html').removeClass(feature).addClass('no-' + feature);
      }
      Modernizr[feature] = value;
    }, feature, value);
  };

  casper.start(casper.cli.get("url"));
  casper.on('page.error', function (msg) {
    if (!ignoreError(msg)) {
      this.die("Page error " + msg, 1);
    } else {
      this.test.comment("Ignoring page error: " + msg);
    }
  });

  waitForLoad();

  casper.then(function () {
    // verify a stylsheet loaded
    this.test.assertTrue(
      this.evaluate(function () {
        return document.styleSheets.length > 1;
      }), "custom stylesheet loaded"
    );
    // verify icon resources loaded
    assertResource('gomap-icons.woff');
    assertResource('texture-noise.png');
    assertResource('modernizr.js');
    assertModernizrDetected('geolocation');
    assertModernizrDetected('ioswebapp');
    assertModernizrDetected('csstransforms3d');
    assertModernizrDetected('csstransitions');
    assertModernizrDetected('touch');
    assertModernizrDetected('ipad');
    assertModernizrDetected('iphone');
    assertModernizrDetected('ipod');
    assertModernizrDetected('svg');
    assertModernizrDetected('useoffcanvasmenu');
    fakeModernizr('useoffcanvasmenu', false);

    this.evaluate(function () {
      window.console = null;
      jQuery.fx.off = true;
      jQuery.support.transition = null;
    });
    shouldSee(".findme");
    shouldSee(".fit");
    this.test.comment("options dropdown toggle");
    shouldSee(WIDE_DROPDOWN_CTRL);
    shouldNotSee(DROPDOWN);
    shouldNotSee(PHONE_DROPDOWN_CTRL);
    this.click(WIDE_DROPDOWN_CTRL);
    shouldSee(DROPDOWN);
    shouldSee(".sat-ctrl");
    shouldSee(DROPDOWN_REVERSE);
    shouldNotBeSelected(BIKE_LAYER);

    fakeModernizr('useoffcanvasmenu', true);
    shouldSee("#nav");
    shouldNotSee(DROPDOWN);
    fakeModernizr('useoffcanvasmenu', false);

    this.click(BIKE_LAYER);
    shouldBeSelected(BIKE_LAYER);
    shouldNotSee(DROPDOWN);
    shouldNotSee(CONTROLS);

    // presence of distance
    this.test.comment("distance display");
    shouldSee(DISTANCE);
    textShouldBe(DISTANCE, "0 mi");
    this.click(DISTANCE);
    textShouldBe(DISTANCE, "0 km");
    this.test.comment("undo button");
    shouldSee(ADDTHIS);
    shouldSee(WIDE_UNDO);
    shouldNotSee(PHONE_UNDO);
    shouldSee(TOP_CLEAR);
    shouldSee(TOP_REVERSE);
    // presence of 4 middle buttons
    this.test.comment("route type toggles");
    var btns = [WALK, BIKE, DRIVE, DIRECT];
    btns.forEach(function (btn) { shouldSee(btn); });
    this.click(DIRECT);
    shouldBeSelected(DIRECT);
    shouldNotBeSelected(WALK);
    this.click(WALK);
    shouldBeSelected(WALK);
    shouldNotBeSelected(DIRECT);

    this.test.comment("share button");
    shouldNotSee(SHARE);
    shouldSee(SHARE_BTN);
    this.click(SHARE_BTN);
    shouldSee(SHARE);
    this.click(SHARE_BTN);
    shouldNotSee(SHARE);

    this.test.comment("feedback");
    shouldNotSee(FEEDBACK_IFRAME);
    this.click(WIDE_DROPDOWN_CTRL);
    this.click("li[data-target='#feedbackModal']");
    shouldSee(FEEDBACK_IFRAME);
    this.test.assertEvalEquals(function() {
      return jQuery("iframe[name=feedback]").attr('src');
    }, 'https://docs.google.com/forms/d/1_vbxe2YK2ECW5WUOzculeGRQTMzQneXf_yOI14WZ8ZI/viewform?embedded=true');
    this.click("#feedbackModal .close");
    shouldNotSee(FEEDBACK_IFRAME);
    this.evaluate(function () {
      jQuery("iframe[name=feedback]").remove();
    });
  });

  casper.waitFor(function () {
    return this.evaluate(function (share) {
      return jQuery(share).is(":visible");
    }, SHARE_BTN);
  });

  casper.then(function () {
    this.test.comment("share button");
    shouldSee(SHARE_BTN);
    this.click(SHARE_BTN);
    shouldSee(SHARE);
    shouldSee(TINYURL);
    this.test.comment("waiting for URL build to finish");
    if (casper.cli.get("remote")) {
      var apikey = this.evaluate(function () {
        return window.OTGM.apikey;
      });
      this.test.assertEquals(apikey, "AIzaSyAKoFgC-xva6WIPbLlkMUpWHVzBHkn5QjM", "apikey");
    }
  });

  casper.waitFor(function () {
    return this.evaluate(function () {
      return jQuery("#emailLink").val().indexOf("http://onthegomap.com") === 0;
    });
  });

  casper.then(function () {
    this.click(TINYURL);
    this.test.comment("waiting for URL shortener to respond");
  });

  casper.waitFor(function () {
    return this.evaluate(function () {
      return jQuery("#emailLink").val().indexOf("http://onthegomap.com") === -1;
    });
  });

  casper.then(function () {
    var compressed = this.evaluate(function () {
      return jQuery("#emailLink").val();
    });
    this.test.comment("Google URL shortener responded: " + compressed);
    this.test.assertTrue(
      compressed.indexOf("http://goo.gl") === 0,
      "URL after shortening was " + compressed
    );
    this.click(SHARE_BTN);
  });

  casper.then(function () {
    this.test.comment("geocoding - center on battery park");
    this.evaluate(function () {
      var evt = new jQuery.Event("keypress");
      evt.keyCode = 13;
      jQuery(".geocode:visible").val("battery park ny");
      jQuery(".geocode:visible").trigger(evt);
    });
  });

  // wait for map to center on battery park
  casper.waitFor(function () {
    return this.evaluate(function () {
      return window.map.getBounds().contains(new google.maps.LatLng(40.703259, -74.016931));
    });
  });

  casper.then(function () {
    this.test.comment("log exception");
    ignoreErrors = true;
    this.evaluate(function () {
      window.gaargs = null;
      window.gaorig = window.ga;
      window.ga = function () {
        window.gaargs = [].slice.apply(arguments);
      };
      setTimeout(function () { throw new Error("message"); }, 0);
    });
  });

  casper.waitFor(function () {
    return this.evaluate(function () {
      return window.gaargs !== null;
    });
  });

  casper.then(function () {
    ignoreErrors = false;
    var gaargs = this.evaluate(function () { return window.gaargs; });
    this.test.assertTrue(/send,event,error,script,[%a-z0-9]*\|running\|Script error.\|\|0,[0-9]*/.test(gaargs),
      "ga args: " + gaargs);
    this.evaluate(function () { window.ga = window.gaorig; });
  });

  casper.then(function () {
    this.test.comment("iphone screen");
    this.viewport(320, 480);
    shouldSee(PHONE_UNDO);
    shouldSee(PHONE_UNDO);
    shouldSee(PHONE_DROPDOWN_CTRL);
    shouldNotSee(WIDE_UNDO);
    shouldNotSee(TOP_CLEAR);
    shouldNotSee(WIDE_DROPDOWN_CTRL);
    shouldNotSee(TOP_REVERSE);
    shouldNotSee(SHARE_BTN);
    shouldSee(ADDTHIS);

    this.click(PHONE_DROPDOWN_CTRL);
    shouldSee(DROPDOWN);

    this.test.comment("share button on iphone");
    shouldNotSee(SHARE);
    this.click(SHARE_LI);
    shouldSee(SHARE);
    this.click(PHONE_DROPDOWN_CTRL);
  });


  casper.then(function () {
    this.test.comment("disable geolocation then reload the page");
    this.options.clientScripts = [];
  });

  casper.thenOpen(casper.cli.get("url"));

  waitForLoad();

  casper.then(function () {
    shouldNotSee(".findme");
    shouldSee(".fit");
    this.test.assertEquals(
      this.fetchText(DISTANCE),
      "0 km", "text of distance should be persisted");
    shouldBeSelected(BIKE_LAYER);
  });

  casper.run(function () {
    this.test.done();
    this.test.renderResults(true);
  });
}());