/*global iframe, store*/
(function () {
  "use strict";
  var ROAD = ".road-ctrl";
  var SATELLITE = ".sat-ctrl";
  var TERRAIN = ".terrain-ctrl";
  var BIKE_LAYER = ".bikelayer-ctrl";
  var ABOUT = 'li a:contains("About")';
  var FEEDBACK = 'li a:contains("Feedback")';
  var SHOW_ELEVATION = '.show-elevation';
  var FINDME = '.findme:visible';
  var ABOUT_DIALOG = '#aboutModal';
  var ABOUT_DIALOG_CLOSE = '#aboutModal .close';
  var FEEDBACK_DIALOG = '#feedbackModal';
  var FEEDBACK_DIALOG_CLOSE = '#feedbackModal .close';
  var DISTANCE = ".distance";
  var UNDO = ".undo-ctrl:visible";
  var TOP_REVERSE = "button.reverse-ctrl";
  var DROPDOWN_REVERSE = "li.reverse-ctrl";
  var TOP_CLEAR = "button.clear-ctrl";
  var SHARE_CTRL = "button.share";
  var SHARE_DROPDOWN = "#options-dropdown li.share";
  var SHARE = "#show-link";
  var TINYURL = "#short-url input";
  var DROPDOWN_CTRL = ".dropdown-control:visible";
  var DROPDOWN = "#options-dropdown";
  var CONTROLS = "#controls";
  var WALK = ".walk";
  var FIT = ".fit:visible";
  var BIKE = ".bike";
  var DRIVE = ".drive";
  var DIRECT = ".direct";
  var CLEAR_ROUTE = '.clear-ctrl';
  var WARNING = '.warning';
  var BRANDING = '.branding';
  var EMBED_TO_SHARE = '.embed-to-share';
  var GEOCODE_BIG = '.input-append .geocode';
  var GEOCODE_SMALL = '#nav .geocode';
  var EMAIL = "#emailLink";
  var $$ = window.jQuery;
  var $iframe = $$("iframe[name=fixture]");
  // save reference to jQuery object of parent window

  module("Control Tests", {
    setup: function () {
      store.clear();
      $iframe.width(1024);
      $iframe.height(768);
    }
  });

  iframe.test("Map is there", function (frame, $, map) {
    ok(map);
  });

  iframe.test("Controls built from hidden div", function (frame, $, map, model, data) {
    ok(!$(CONTROLS).length, "control div should be removed");
    data.each(function () {
      // all children of old div should be inserted into DOM (except branding,
      // that is only for embedded)
      if (!$(this).hasClass("branding")) {
        ok($.contains(frame.document.documentElement, this), $(this).html);
      }
    });
  });

  var numSocialButtonsShouldBe = function (frame, $, n) {
    var cb = frame.scriptsRequested['addthis_widget.js'];
    if (cb) {
      strictEqual($(".addthis_toolbox:visible").length, 0, "number of social areas before callback");
      cb();
      delete frame.scriptsRequested['addthis_widget.js'];
    }
    strictEqual($(".addthis_toolbox:visible").length, n, "number of social areas");
  };

  var shouldNotShowSocialButtons = function (frame, $) {
    numSocialButtonsShouldBe(frame, $, 0);
  };

  var checkButtonsThatShouldAlwaysShowUp = function (frame, $) {
    var control = $(DROPDOWN_CTRL);
    var options = $(DROPDOWN);
    ok($(DISTANCE).is(":visible"), "Distance visible");
    ok(control.is(":visible"), "Dropdown control visible");
    ok(!options.is(":visible"), "Dropdown options start invisible");
    strictEqual($(DISTANCE).text(), "0 mi");
    var btns = [WALK, BIKE, DRIVE, DIRECT];
    $.each(btns, function (idx, btn) {
      ok($(btn).is(":visible"), btn + " visible");
    });
    ok($(UNDO).is(":visible"), "undo visible");
    ok($(FINDME).is(":visible"),
      "Go to my location visible");
    ok(!$(SHARE).is(":visible"), "share not visible");
    ok(!$(BRANDING).is(":visible"), "branding not visible");
    ok(!$(EMBED_TO_SHARE).is(":visible"), "new tab icon not visible");
    strictEqual($(FIT).length, 1, "num fits visible");
    ok($(".undo-ctrl").hasClass("disabled"), "undo starts disabled");
    ok($(".fit").hasClass("disabled"), "fit starts disabled");
    ok($(".clear-ctrl").hasClass("disabled"), "clear starts disabled");
  };

  iframe.asyncTest("iPhone controls", function (frame, $) {
    var dd = $(DROPDOWN_CTRL);
    frame.fakeModernizr('useoffcanvasmenu', true);
    $.fx.off = true;
    $iframe.width(320);
    $iframe.height(480);

    setTimeout(function () {
      checkButtonsThatShouldAlwaysShowUp(frame, $);

      // responsive...
      ok(!$(TOP_CLEAR).is(":visible"), "top-level clear is not visible");
      ok(!$(TOP_REVERSE).is(":visible"), "top-level reverse is not visible");
      ok(!$(SHARE_CTRL).is(":visible"), "share not visible");
      ok(!$(GEOCODE_BIG).is(":visible"), "big geocode not visible");
      dd.click();
      ok($(GEOCODE_SMALL).is(":visible"), "geocode small visible");
      dd.click();
      start();
    }, 100);
  });

  iframe.asyncTest("Desktop controls", function (frame, $) {
    var dd = $(DROPDOWN_CTRL);
    frame.fakeModernizr('useoffcanvasmenu', false);
    $.fx.off = true;
    $iframe.width(1024);
    $iframe.height(768);

    setTimeout(function () {
      checkButtonsThatShouldAlwaysShowUp(frame, $);

      // responsive...
      ok($(TOP_CLEAR).is(":visible"), "top-level clear is visible");
      ok($(TOP_REVERSE).is(":visible"), "top-level reverse is visible");
      ok($(SHARE_CTRL).is(":visible"), "share is visible");
      // TODO
      ok(!$(SHARE_DROPDOWN).is(":visible"));
      ok($(GEOCODE_BIG).is(":visible"), "geocode big visible");
      dd.click();
      ok(!$(GEOCODE_SMALL).is(":visible"), "geocode small not visible");
      dd.click();
      start();
    }, 100);
  });

  iframe.test("Dropdown control toggling", function (frame, $) {
    var control = $(DROPDOWN_CTRL);
    var options = $(DROPDOWN);
    $.fx.off = true;

    // dropdown behavior
    strictEqual(control.length, 1,
      "Dropdown contol is there");
    strictEqual(options.length, 1,
      "Dropdown options are there");
    ok(!options.is(":visible"),
      "Dropdown options start invisible");
    control.click();
    ok(options.is(":visible"),
      "Clicking control makes options visible");
    ok($(ROAD).is(":visible"),
      "Road map control visible on dropdown");
    ok($(SATELLITE).is(":visible"),
      "Satellite map control visible on dropdown");
    ok($(TERRAIN).is(":visible"),
      "Terrrain map control visible on dropdown");
    ok($(BIKE_LAYER).is(":visible"),
      "Bike overlay control visible on dropdown");
    ok($(DROPDOWN_REVERSE).is(":visible"),
      "Reverse control visible on dropdown");
    control.click();
    ok(!options.is(":visible"),
      "Clicking control again makes options invisible");

    control.click();
    ok(options.is(":visible"));
    $('.divider').click();
    $('.divider').trigger('mousedown');
    ok(options.is(":visible"),
      "clicking divider should not close dropdown");
    $(CLEAR_ROUTE).click();
    ok(!options.is(":visible"),
      "Clicking options again makes options invisible");
  });

  iframe.test("Share route 'show'/'shown'/'close' event", function (frame, $) {
    expect(2);
    var control = $(SHARE_CTRL);
    var $dd = $("#show-link");
    $.fx.off = true;

    $dd.on("show", function () {
      ok(true, "show");
    });
    $dd.on("close", function () {
      ok(true, "close");
    });
    // dropdown behavior
    control.click();
    $dd.off("show shown");
    control.click();
  });

  iframe.test("Share route toggling", function (frame, $) {
    var ddCtrl = $(SHARE_DROPDOWN);
    var control = $(SHARE_CTRL);
    var show = $(SHARE);
    $.fx.off = true;

    // dropdown behavior
    strictEqual(control.length, 1,
      "Dropdown contol is there");
    strictEqual(show.length, 1,
      "Show link is there");
    ok(!show.is(":visible"),
      "Show link start invisible");
    control.click();
    ok(show.is(":visible"),
      "Clicking control makes show visible");
    control.click();
    ok(!show.is(":visible"),
      "Clicking control again makes show invisible");

    ddCtrl.click();
    ok(show.is(":visible"),
      "clicking link from dropdown makes visible");
    $("#show-link").click();
    ok(show.is(":visible"),
      "show link still visible after click it");

    $(UNDO).trigger('mousedown');
    ok(!show.is(":visible"),
      "Clicking clear route makes show link invisible");

    control.click();
    ok(show.is(":visible"),
      "Clicking control makes show visible");
    $("#show-link .close").click();
    ok(!show.is(":visible"),
      "Clicking 'x' closes");
  });

  $$.each(['#nav', '#options-dropdown'], function (idx, parent) {
    iframe.test("Map type control " + parent, function (frame, $, map) {
      var google = frame.google,
      control = $(DROPDOWN_CTRL),
      road = $(parent + " " + ROAD),
      satellite = $(parent + " " + SATELLITE),
      terrain = $(parent + " " + TERRAIN),
      bike = $(parent + " " + BIKE_LAYER),
      stage,
      shouldOnlyBeChecked = function () {
        var args = $.makeArray(arguments);
        $.each([road, terrain, satellite, bike], function (idx, arg) {
          if ($.inArray(arg, args) > -1) {
            ok(arg.hasClass("checked"), arg.attr("class") + " should be selected " + stage);
          } else {
            ok(!arg.hasClass("checked"), arg.attr("class") + " should be NOT selected " + stage);
          }
        });
      },
      mapTypeShouldBe = function (type) {
        strictEqual(map.getMapTypeId(), type, "map type " + stage);
      },
      shouldSeeBikeLayer = function () {
        strictEqual(iframe.bikeLayers.length, 1, "Should be a bike layer " + stage);
      },
      shouldNotSeeBikeLayer = function () {
        strictEqual(iframe.bikeLayers.length, 0, "Should be no bike layer " + stage);
      };

      stage = "on initializtion";
      mapTypeShouldBe(google.maps.MapTypeId.ROADMAP);
      shouldNotSeeBikeLayer();
      shouldOnlyBeChecked(road);

      stage = "on click satellite";
      control.click();
      satellite.click();
      shouldOnlyBeChecked(satellite);
      mapTypeShouldBe(google.maps.MapTypeId.HYBRID);
      shouldNotSeeBikeLayer();

      stage = "on click terrain";
      control.click();
      terrain.click();
      shouldOnlyBeChecked(terrain);
      mapTypeShouldBe(google.maps.MapTypeId.TERRAIN);
      shouldNotSeeBikeLayer();

      stage = "on click bike";
      control.click();
      bike.click();
      shouldOnlyBeChecked(terrain, bike);
      mapTypeShouldBe(google.maps.MapTypeId.TERRAIN);
      shouldSeeBikeLayer();

      stage = "on click road";
      control.click();
      road.click();
      shouldOnlyBeChecked(road, bike);
      mapTypeShouldBe(google.maps.MapTypeId.ROADMAP);
      shouldSeeBikeLayer();

      // ensure external change reflected in custom controls
      stage = "on external map type change";
      map.setMapTypeId(google.maps.MapTypeId.TERRAIN);
      shouldOnlyBeChecked(terrain, bike);

      // reset
      road.click();
      bike.click();
      iframe.bikeLayers = [];
    });
  });

  iframe.test("Direction type control", function (frame, $, map, model) {
    var google = frame.google;
    var walk = $(WALK);
    var bike = $(BIKE);
    var drive = $(DRIVE);
    var direct = $(DIRECT);
    var stage;
    var shouldOnlyBeChecked = function () {
      var args = $.makeArray(arguments);
      $.each([walk, bike, drive, direct], function (idx, arg) {
        if ($.inArray(arg, args) > -1) {
          ok(arg.hasClass("checked"), arg.attr("class") + " should be selected " + stage);
        } else {
          ok(!arg.hasClass("checked"), arg.attr("class") + " should be NOT selected " + stage);
        }
      });
    };
    var travelModeShouldBe = function (type) {
      strictEqual(model.getTravelMode(), type, "travel mode " + stage);
    };

    stage = "initial";
    shouldOnlyBeChecked(walk);
    travelModeShouldBe(google.maps.TravelMode.WALKING);

    stage = "click bike";
    bike.click();
    shouldOnlyBeChecked(bike);
    travelModeShouldBe(google.maps.TravelMode.BICYCLING);

    stage = "click drive";
    drive.click();
    shouldOnlyBeChecked(drive);
    travelModeShouldBe(google.maps.TravelMode.DRIVING);

    stage = "click direct";
    direct.click();
    shouldOnlyBeChecked(direct);
    travelModeShouldBe(null);

    stage = "click walk";
    walk.click();
    shouldOnlyBeChecked(walk);
    travelModeShouldBe(google.maps.TravelMode.WALKING);
  });


  $$.each(['#nav', '#options-dropdown'], function (idx, parent) {
    iframe.asyncTest("About dialog " + parent, function (frame, $) {
      frame.fakeModernizr('useoffcanvasmenu', parent === "#nav");
      var $control = $(DROPDOWN_CTRL);
      var $about = $(parent + " " + ABOUT);
      var $aboutDialog = $(ABOUT_DIALOG);
      var $closeAbout = $(ABOUT_DIALOG_CLOSE);

      // disable transtions
      $.support.transition = null;

      strictEqual($closeAbout.length, 1, "close matches");

      ok(!$aboutDialog.is(":visible"), "about dialog should start invisible");
      ok(!$about.is(":visible"), "show about button starts invisible");

      $control.click();
      ok(!$aboutDialog.is(":visible"), "after click dropdown, about dialog still invisible");
      ok($about.is(":visible"), "after click dropdown, about control shows up");

      $about.click();
      setTimeout(function () {
        ok($aboutDialog.is(":visible"), "after click 'about', about dialog becomes visible");

        ok(!$about.is(":visible"), "after click 'about', about control gets hidden");

        // verify contents of about dialog
        (function () {
          var assertLinkTo = function (contains, href) {
            var link = $aboutDialog.find('a:contains("' + contains + '")');
            ok(link.is(":visible"), "'" + contains + "' link should be visible");
            strictEqual(link.attr('href'), href, contains + " URL should be " + href);
          };
          assertLinkTo("Entypo", "http://www.entypo.com/");
          assertLinkTo("IcoMoon", "http://keyamoon.com/icomoon/");
          assertLinkTo("Font Awesome", "http://fontawesome.io");
          assertLinkTo("CC BY 3.0", "http://creativecommons.org/licenses/by/3.0/");
          assertLinkTo("CC BY-SA 3.0", "http://creativecommons.org/licenses/by-sa/3.0/");
          assertLinkTo("Bootstrap", "http://twitter.github.com/bootstrap");
          assertLinkTo("Apache", "http://www.apache.org/licenses/LICENSE-2.0");
          if (navigator.standalone) {
            $("html").addClass("no-ioswebapp").removeClass("ioswebapp");
          }
          frame.fakeModernizr("ioswebapp", false);
          ok($(".iosadd").is(":visible"), "iosadd should be visible");
          frame.fakeModernizr("ioswebapp", true);
          ok(!$(".iosadd").is(":visible"), "iosadd should not be visible");
        }());

        $closeAbout.click();
        ok(!$aboutDialog.is(":visible"), "after click 'close', about dialog becomes invisible");

        // make sure iOS gesture aliases work
        $control.click();
        $about.trigger('click');
        ok($aboutDialog.is(":visible"), "after click 'about', about dialog becomes visible");
        $('.modal-backdrop').trigger('touchstart').trigger('mousedown');
        ok(!$aboutDialog.is(":visible"), "after backdrop mousedown, about dialog should be invisible");

        // make sure give feedback button closes this dialog and opens other one
        $control.click();
        $about.trigger('click');
        var $feedback = $('#aboutModal a:contains("Feedback"):visible');
        strictEqual($feedback.length, 1, "num feedback results");
        $feedback.click();
        ok(!$aboutDialog.is(":visible"), "after click feedback, about closes");
        ok($(FEEDBACK_DIALOG).is(":visible"), "after click feedback, feedback opens");
        $(FEEDBACK_DIALOG_CLOSE).click();
        ok(!$(FEEDBACK_DIALOG).is(":visible"), "after close feedback, feedback closes");
        start();
      }, 100);
    });

    iframe.asyncTest("Feedback dialog " + parent, function (frame, $) {
      frame.fakeModernizr('useoffcanvasmenu', parent === "#nav");
      var $control = $(DROPDOWN_CTRL);
      var $feedback = $(parent + " " + FEEDBACK);
      var $feedbackModal = $(FEEDBACK_DIALOG);
      var $closeFeedback = $(FEEDBACK_DIALOG_CLOSE);

      // disable transtions
      $.support.transition = null;

      strictEqual($closeFeedback.length, 1, "close matches");

      ok(!$feedbackModal.is(":visible"), "feedback dialog should start invisible");
      ok(!$feedback.is(":visible"), "show feedback button starts invisible");

      $control.click();
      ok(!$feedbackModal.is(":visible"), "after click dropdown, feedback dialog still invisible");
      ok($feedback.is(":visible"), "after click dropdown, feedback control shows up");

      $feedback.click();
      ok(!$feedback.is(":visible"), "after click 'feedback', feedback control gets hidden");
      ok($(FEEDBACK_DIALOG).is(":visible"), "after click feedback, feedback opens");

      $closeFeedback.click();
      ok(!$feedbackModal.is(":visible"), "after click 'close', about dialog becomes invisible");
      setTimeout(start, 0);
    });
  });


  iframe.test("Warning display", function (frame, $) {
    var $aboutDialog = $(ABOUT_DIALOG);
    var $closeAbout = $(ABOUT_DIALOG_CLOSE);
    var $warning = $(WARNING);

    $.support.transition = null;

    ok(!$warning.is(":visible"), "warning not displayed by default");
    ok(!$aboutDialog.is(":visible"), "about dialog starts not visible");
    $warning.find("a").click();
    ok($aboutDialog.is(":visible"), "click warning opens about dialog");
    $closeAbout.click();
    ok(!$aboutDialog.is(":visible"), "about dialog closed");
  });

  $$.each([
    {
      name: 'About',
      controlSelector: "#options-dropdown " + ABOUT,
      dialogSelector: ABOUT_DIALOG,
      closeSelector: ABOUT_DIALOG_CLOSE
    },
    {
      name: 'Feedback',
      controlSelector: "#options-dropdown " + FEEDBACK,
      dialogSelector: FEEDBACK_DIALOG,
      closeSelector: FEEDBACK_DIALOG_CLOSE
    }
  ], function (idx, testData) {
    iframe.test(name + " modal dialog responsive styling", function (frame, $) {
      var $control = $(DROPDOWN_CTRL);
      var $dialogOpener = $(testData.controlSelector);
      var $modalDialog = $(testData.dialogSelector);
      var $closeDialog = $(testData.closeSelector);
      var $modalBody = $modalDialog.find(".modal-body");
      var bottom = function () {
        return $modalDialog.position().top + $modalDialog.height();
      };

      // disable transtions
      $.support.transition = null;
      $control.click();
      $dialogOpener.click();
      ok($modalDialog.is(":visible"), "after click 'about', about dialog becomes visible");

      var previousHtml = $modalBody.html();
      try {
        // put a bunch of junk in the dropdown dialog
        $modalBody.html(new Array(1000).join("test<br>"));
        // test responsive styling - big screen...
        $iframe.height(800);
        strictEqual($modalBody.height(), 400, "height on big screen should be 400");

        // check all 10px increments from 300 to 800 px, dialog should never be
        // taller than window
        for (var i = 320; i < 800; i += 10) {
          $iframe.height(i - 1);
          ok(bottom() < (i - 4), "at " + (i - 1) + "p height, about dialog was " +
            bottom() + "px");
          $iframe.height(i + 1);
          ok(bottom() < (i - 4), "at " + (i + 1) + "p height, about dialog was " +
            bottom() + "px");
        }
      } finally {
        $iframe.height(1000);
        $modalBody.html(previousHtml);
        $closeDialog.click();
      }
    });
  });

  iframe.test("direct routing hooked up", function (frame, $, map) {
    var $distance = $(DISTANCE);
    var google = frame.google;
    var click = function (lat, lng) {
      google.maps.event.trigger(map, 'singleclick', {
        latLng: new google.maps.LatLng(lat, lng)
      });
    };
    $(DIRECT).click();

    strictEqual($distance.text(), "0 mi");
    $distance.click();
    strictEqual($distance.text(), "0 km");
    $distance.click();
    strictEqual($distance.text(), "0 mi");
    click(0, 0);
    strictEqual($distance.text(), "0 mi");
    click(0, 1);
    strictEqual($distance.text(), "69.2 mi");
    $distance.click();
    strictEqual($distance.text(), "111.3 km");
    click(0, 2);
    strictEqual($distance.text(), "222.6 km");
    click(0, 3);
    strictEqual($distance.text(), "334 km");

    $(UNDO).click();
    strictEqual($distance.text(), "222.6 km");
    $(TOP_REVERSE).click();
    strictEqual($distance.text(), "222.6 km");

    $(DROPDOWN_CTRL).click();
    $(CLEAR_ROUTE).click();
    strictEqual($distance.text(), "0 km");

    // large-screen buttons
    click(0, 0);
    click(0, 1);
    click(0, 2);
    strictEqual($distance.text(), "222.6 km");
    $(UNDO).click();
    strictEqual($distance.text(), "111.3 km");
    $(TOP_CLEAR).click();
    strictEqual($distance.text(), "0 km");

    click(0, 1);
    click(0, 2);
    $(TOP_REVERSE).click();
    click(0, 0);
    strictEqual($distance.text(), "222.6 km");
    $(UNDO).click();
    strictEqual($distance.text(), "111.3 km");
    $(TOP_REVERSE).click();
    click(0, 3);
    strictEqual($distance.text(), "222.6 km");
    var oldConfirm = frame.confirm;
    frame.confirm = function () { return true; };
    $(frame).trigger('shake');
    frame.confirm = oldConfirm;
    strictEqual($distance.text(), "0 km");
  });

  iframe.test("fit route button", function (frame, $, map) {
    var google = frame.google;
    var click = function (lat, lng) {
      google.maps.event.trigger(map, 'singleclick', {
        latLng: new google.maps.LatLng(lat, lng)
      });
    };
    $(DIRECT).click();
    click(50, 50);
    ok(!map.getBounds().contains(new frame.google.maps.LatLng(50, 50)), "map doesnt fit new point before fit");
    $(FIT).click();
    click(0, 0);
    ok(map.getBounds().contains(new frame.google.maps.LatLng(50, 50)), "map does fit new point after fit");
    ok(!map.getBounds().contains(new frame.google.maps.LatLng(0, 0)), "map doesnt fit 2nd point before 2nd fit");
    $(FIT).click();
    ok(map.getBounds().contains(new frame.google.maps.LatLng(0, 0)), "map does fit 2nd point after 2nd fit");
    $(TOP_CLEAR).click();
  });

  iframe.test("enable buttons when have points", function (frame, $, map) {
    var google = frame.google;
    var click = function (lat, lng) {
      google.maps.event.trigger(map, 'singleclick', {
        latLng: new google.maps.LatLng(lat, lng)
      });
    };
    var shouldBeDisabled = function (value) {
      ok(value === $(".undo-ctrl").hasClass("disabled"), "undo starts disabled");
      ok(value === $(".fit").hasClass("disabled"), "fit starts disabled");
      ok(value === $(".clear-ctrl").hasClass("disabled"), "clear starts disabled");
    };

    shouldBeDisabled(true);

    $(DIRECT).click();
    click(50, 50);

    shouldBeDisabled(false);

    click(51, 51);

    shouldBeDisabled(false);

    $(UNDO).click();
    shouldBeDisabled(false);
    $(UNDO).click();
    shouldBeDisabled(true);
  });

  // reload iframe prior to warning/copyright tests
  iframe.reload();

  iframe.test("Warning displayed when new warning or copyright", function (frame, $, map, model) {
    var $warning = $(WARNING);

    $.support.transition = null;
    $.fx.off = true;

    ok($warning.css('display') === 'none', "warning not displayed by default");

    model.setLegal({
      warnings: ["warning1", "warning2"],
      copyrights: []
    });

    ok($warning.css('display') !== 'none', "warning displayed after model upated with legal");

    model.setLegal({
      warnings: [],
      copyrights: ["copyright"]
    });
    ok($warning.css('display') !== 'none', "warning still displayed after model upated with nonempty legal");

    model.setLegal({
      warnings: [],
      copyrights: []
    });
    ok($warning.css('display') === 'none', "warning hidden after model upated with new empty legal");
  });

  iframe.test("Warnings and copyrights go to about dialog", function (frame, $, map, model) {
    model.setLegal({
      warnings: ["warning1", "warning2"],
      copyrights: ["copyright1", "copyright2"]
    });

    ok($('p:contains("warning1")').length > 0,
      "about dialog contains first warning");

    ok($('p:contains("warning2")').length > 0,
      "about dialog contains second warning");

    ok($('p:contains("copyright1")').length > 0,
      "about dialog contains first copyright");

    ok($('p:contains("copyright2")').length > 0,
      "about dialog contains second copyright");

    model.setLegal({
      warnings: [],
      copyrights: []
    });

    ok($('p:contains("warning")').length === 0,
      "about dialog should not contain warning after update removes it");

    ok($('p:contains("copyright")').length === 0,
      "about dialog should not contain copyright after update removes it");
  });

  iframe.test("Go to my location button", function (frame, $, map) {
    var $findme = $(FINDME);
    var centerShouldBe = function (lat, lng, msg) {
      var center = map.getCenter();
      strictEqual(Math.round(center.lat()), lat, "latitude " + msg);
      strictEqual(Math.round(center.lng()), lng, "longitude " + msg);
    };

    // disable transtions
    $.support.transition = null;

    strictEqual($findme.length, 1, "find me matches");

    ok($findme.is(":visible"), "find me always visible");

    // make sure last request responded to
    frame.OTGM.geolocation.lastSucc = null;

    $findme.click();

    ok(frame.OTGM.geolocation.lastSucc,
      "clicking find me button makes geolocation request");

    frame.OTGM.geolocation.lastSucc({
      coords: {
        latitude: 3,
        longitude: 4
      }
    });

    centerShouldBe(3, 4, "after find me response");
  });

  iframe.test("Show Elevation Button", function (frame, $) {
    var $showElevation = $(SHOW_ELEVATION);

    strictEqual($showElevation.length, 1, "show elevation in DOM");
    frame.fakeModernizr('svg', false);
    ok(!$showElevation.is(":visible"), "elevation NOT button shown without SVG");
    frame.fakeModernizr('svg', true);
    $("html").addClass("no-d3");
    ok(!$showElevation.is(":visible"), "elevation button not shown with SVG but no d3");
    $("html").removeClass("no-d3");
    ok($showElevation.is(":visible"), "elevation button shown with SVG & d3");

    ok(!$("#elevation").is(":visible"));
    $showElevation.click();
    ok($("#elevation").is(":visible"));
    $showElevation.click();
    ok(!$("#elevation").is(":visible"));
  });

  iframe.test("Geocode text box", function (frame, $, map) {
    var lastReq, lastCb;
    var $geocodeBig = $(GEOCODE_BIG);
    var $geocodeSmall = $(GEOCODE_SMALL);
    var bounds = new frame.google.maps.LatLngBounds(
      new frame.google.maps.LatLng(10, 10),
      new frame.google.maps.LatLng(11, 11)
    );
    var noError = function () {
      ok(!$geocodeBig.parent().hasClass('error'), "big no error");
      ok(!$geocodeSmall.parent().hasClass('error'), "big no error");
    };
    frame.google.maps.Geocoder.mockImpl = function (arg, cb) {
      lastCb = cb;
      lastReq = arg;
      frame.OTGM.resetGeocode();
    };
    $.fx.off = true;

    // use small, ensure invoked. on success: text is cleared
    $geocodeBig.val("test");
    var evt = function () {
      var result = new frame.jQuery.Event("keypress");
      result.keyCode = 13;
      return result;
    };
    $geocodeBig.trigger(evt());
    ok(lastCb, "mock impl invoked");
    strictEqual(lastReq.address, "test", "addr past to service");
    noError();

    // make respond
    lastCb([{
      geometry: {
        bounds: bounds
      }
    }], frame.google.maps.GeocoderStatus.OK);
    noError();
    ok(map.getBounds().contains(new frame.google.maps.LatLng(11, 11)),
      "map bounds fit at least response bounds");
    strictEqual($geocodeBig.val(), "", "value after press enter");

    lastCb = null;

    // use small one
    $(DROPDOWN_CTRL).click();
    $geocodeSmall.val("test");
    $geocodeSmall.trigger(evt());
    ok(lastCb, "mock impl invoked");
    noError();

    // make respond
    lastCb([{
      geometry: {
        bounds: bounds
      }
    }], frame.google.maps.GeocoderStatus.OK);
    noError();
    strictEqual($geocodeSmall.val(), "", "value after press enter");
    ok(!$(DROPDOWN).is(":visible"), "response makes dropdown slide up");

    // error 
    $geocodeBig.val("test");
    $geocodeBig.trigger(evt());

    // make respond
    lastCb([], frame.google.maps.GeocoderStatus.ERROR);
    ok($geocodeBig.parent().hasClass("error"), "big on has error");
    ok(!$geocodeSmall.parent().hasClass("error"), "small one doesnt have error");

    // success clears
    lastCb = null;
    $geocodeBig.val("test");
    $geocodeBig.trigger(evt());

    // make respond
    lastCb([{geometry: {bounds: bounds}}], frame.google.maps.GeocoderStatus.OK);
    noError();

    lastCb([], frame.google.maps.GeocoderStatus.ERROR);
    ok($geocodeBig.parent().hasClass("error"), "big one has error");
    var clearevt = new frame.jQuery.Event("keyup");
    $geocodeBig.val("");
    $geocodeBig.trigger(clearevt);
    ok(!$geocodeBig.parent().hasClass("error"), "big one has no error after clearing");

    lastCb = null;
    $geocodeBig.val("");
    $geocodeBig.trigger(evt());
    ok(!lastCb, "service not invoked on empty input");
  });

  var loadIframeAndStartTest = function (iframeHtml, $, map, toRun) {
    var $$newIframe = $$(iframeHtml);
    $$newIframe.one('load', function () {
      var iFrame = $$newIframe[0];
      var newFrame = iFrame.contentWindow;
      var $ = newFrame.jQuery;
      newFrame.scriptsRequested = {};
      newFrame.jQuery.getScript = function (name, callback) {
        newFrame.scriptsRequested[name.replace(/(^.*\/|#.*$)/g, '')] = callback;
        return {
          fail: function () {}
        };
      };
      newFrame.fakeModernizr = function (key, value) {
        if (value) {
          newFrame.$("html").addClass(key).removeClass("no-" + key);
        } else {
          newFrame.$("html").removeClass(key).addClass("no-" + key);
        }
        newFrame.Modernizr[key] = value;
      };
      $("iframe[name=feedback]").remove();
      $(newFrame.document).one('gomapready', function (e, map, model) {
        toRun(map, model, newFrame.$, newFrame);
        $$newIframe.remove();
        store.clear();
        start();
      });
      $(newFrame.document).trigger('isgomapready');
    });
    $$("#qunit-fixture").append($$newIframe);
  };

  var testInIframe = function (frame, $, map, toRun) {
    var control = $(SHARE_CTRL);
    $.fx.off = true;
    control.click();
    store.clear();
    var embed = $("#embedLink").val().replace("http://onthegomap.com/", frame.location.href.split("?")[0]);
    ok($(TINYURL).is(":enabled"));
    control.click();
    loadIframeAndStartTest(embed, $, map, toRun);
  };

  var getShareUrl = function (frame, $) {
    var control = $(SHARE_CTRL);
    $.fx.off = true;
    control.click();
    store.clear();
    var shareURL = $("#emailLink").val().replace("http://onthegomap.com/", frame.location.href.split("?")[0]);
    ok($(TINYURL).is(":enabled"));
    control.click();
    return shareURL;
  };

  var testShared = function (frame, $, map, toRun) {
    var shareURL = getShareUrl(frame, $);
    loadIframeAndStartTest("<iframe src=" + shareURL + "></iframe>", $, map, toRun);
  };

  iframe.asyncTest("Shorten URL", function (frame, $$, map) {
    // UI tests
    var ERROR = "Short URL error";
    var IN_PROGRESS = "Short URL...";
    var NORMAL = "Short URL";
    var $control = $$(SHARE_CTRL);
    var $email = $$(EMAIL);
    var $shorten = $$(TINYURL);
    var label = function () {
      return $$("#short-url-label").text();
    };
    $$.fx.off = true;
    frame.gapi = {};

    // hidden at first
    $control.click();
    setTimeout(function () {
      var orig = $email.val();
      ok($email.is(":visible"), "email is visible");
      ok(!$shorten.is(":visible"), "shorten starts invisible");

      // shown after load successful
      frame.gapi.client = {};
      var lastCb;
      frame.gapi.client.setApiKey = function () {};
      frame.gapi.client.load = function (a, b, callback) {
        frame.gapi.client.urlshortener = {};
        frame.gapi.client.urlshortener.url = {};
        frame.gapi.client.urlshortener.url.insert = function () {
          return {
            execute: function (cb) {
              lastCb = cb;
            }
          };
        };
        callback();
      };
      frame.OTGMgapiclientload();
      ok($shorten.is(":visible"), "shorten visible after load");

      // click -> in progress
      strictEqual(label(), NORMAL, "original label");
      $shorten.click();
      strictEqual(label(), IN_PROGRESS, "label after click");

      // success -> replaces URL
      lastCb({id: "short URL"});
      strictEqual(label(), NORMAL, "label after response");
      strictEqual($email.val(), "short URL", "link after response");

      // unclick -> uses old
      $shorten.click();
      strictEqual(label(), NORMAL, "label after response");
      strictEqual($email.val(), orig, "link after unclick");

      // close then reopen and error
      $control.click();
      map.setCenter(new frame.google.maps.LatLng(10, 10));
      $control.click();
      $shorten.click();
      lastCb({error: true});
      strictEqual(label(), ERROR, "label after error");

      // close then request reopen and get error
      $control.click();
      map.setCenter(new frame.google.maps.LatLng(11, 11));
      $control.click();
      $shorten.click();
      $control.click();
      $control.click();
      lastCb({error: true});
      strictEqual(label(), NORMAL, "label after error but reopened, should be ignored");
      $control.click();
      start();
    }, 100);
  });

  iframe.asyncTest("Use the embedded iframe code with no route", function (frame, $$, oldMap, oldModel) {
    oldMap.setMapTypeId(frame.google.maps.MapTypeId.HYBRID);
    $$("#options-dropdown " + BIKE_LAYER).click();
    $$(DISTANCE).click();
    ok($$(DISTANCE).text().search(" km") > -1, "orig map starts using KM");
    oldModel.setLegal({
      warnings: ["warning 1", "warning 2"],
      copyrights: ["copyright 1", "copyright 2"]
    });
    var shareUrl = getShareUrl(frame, $$);
    testInIframe(frame, $$, oldMap, function (map, model, $, newFrame) {
      $.fx.off = true;
      strictEqual(
        map.getMapTypeId(),
        newFrame.google.maps.MapTypeId.HYBRID,
        "map type of new map"
      );
      strictEqual($(".map-btn").length, 0, "no buttons should be visible");
      ok($(DISTANCE).is(":visible"), "distance displayed");
      ok($(WARNING).is(":visible"), "warning displayed");
      strictEqual($(DISTANCE).text(), "0 km", "km displayed");
      $(DISTANCE).click();
      strictEqual($(DISTANCE).text(), "0 mi", "toggles to mi");
      ok(model.getBikeLayer(), "bike layer displayed");
      strictEqual(model.getLegal().warnings[0], "warning 1");
      strictEqual(model.getLegal().warnings[1], "warning 2");
      strictEqual(model.getLegal().copyrights[0], "copyright 1");
      strictEqual(model.getLegal().copyrights[1], "copyright 2");
      ok($(EMBED_TO_SHARE).is(":visible"), "branding displayed");
      strictEqual($(EMBED_TO_SHARE + " a").attr('href'), shareUrl);
      ok(!$(FINDME).is(":visible"), "find me visible");
      ok(!$(FIT).is(":visible"), "fit fixible");
      shouldNotShowSocialButtons(newFrame, $);
    });
  });

  iframe.asyncTest("Test shared", function (frame, $$, oldMap) {
    $$(DIRECT).click();
    var click = function (lat, lng) {
      frame.google.maps.event.trigger(oldMap, 'singleclick', {
        latLng: new frame.google.maps.LatLng(lat, lng)
      });
    };
    click(0, 4);
    click(0, 5);
    testShared(frame, $$, oldMap, function (map, model, $, newFrame) {
      newFrame.fakeModernizr('geolocation', false);
      $.fx.off = true;
      strictEqual(
        map.getMapTypeId(),
        newFrame.google.maps.MapTypeId.HYBRID,
        "map type of new map"
      );
      strictEqual($(".map-btn").length, 3, "buttons added");
      ok($(DISTANCE).is(":visible"), "distance displayed");
      ok(!$(FINDME).is(":visible"), "find me visible");
      ok($(FIT).is(":visible"), "fit fixible");
      var reqMade = 0;
      newFrame.OTGM.geolocation = {
        getCurrentPosition: function () { reqMade++; },
        watchPosition: function () {}
      };
      newFrame.fakeModernizr('geolocation', true);
      ok($(FINDME).is(":visible"), "with geolocation, find me visible");
      newFrame.OTGM.geolocation.lastSucc = null;
      $(FINDME).click();
      ok($(BRANDING).is(":visible"), "branding displayed");
      strictEqual($(BRANDING + " a").attr('href'), "http://onthegomap.com");
      ok(reqMade === 1,
        "clicking find me button makes geolocation request");
      map.setCenter(new newFrame.google.maps.LatLng(50, 50));
      ok(!map.getBounds().contains(new newFrame.google.maps.LatLng(0, 4)),
        "route doesn't fit before click fit");
      $(FIT).click();
      ok(map.getBounds().contains(new newFrame.google.maps.LatLng(0, 4)),
        "clicking fit moves center");
      $$(CLEAR_ROUTE).click();
      shouldNotShowSocialButtons(newFrame, $);
    });
  });

  iframe.asyncTest("Use the embedded iframe code with a route", function (frame, $$, oldMap, model) {
    oldMap.setMapTypeId(frame.google.maps.MapTypeId.HYBRID);
    model.setUnit('mi');
    var click = function (lat, lng) {
      frame.google.maps.event.trigger(oldMap, 'singleclick', {
        latLng: new frame.google.maps.LatLng(lat, lng)
      });
    };
    $$(DIRECT).click();
    click(0, 4);
    click(0, 5);
    strictEqual($$(DISTANCE).text(), "69.2 mi");
    strictEqual(model.getUnit(), "mi");
    testInIframe(frame, $$, oldMap, function (map, model, $) {
      $.fx.off = true;
      strictEqual(model.getUnit(), "mi");
      strictEqual($(DISTANCE).text(), "69.2 mi");
      ok(map.getBounds().contains(new frame.google.maps.LatLng(0, 4)),
        "map bounds should fit 0, 4");
      ok(map.getBounds().contains(new frame.google.maps.LatLng(0, 5)),
        "map bounds should fit 0, 5");
    });
  });

  iframe.asyncTest("Test overriding distance", function (frame, $$, oldMap, oldModel) {
    oldModel.setUnit('mi');
    var click = function (lat, lng) {
      frame.google.maps.event.trigger(oldMap, 'singleclick', {
        latLng: new frame.google.maps.LatLng(lat, lng)
      });
    };
    $$(DIRECT).click();
    click(0, 4);
    click(0, 5);
    oldModel.setMeters(1609);
    testInIframe(frame, $$, oldMap, function (map, model, $) {
      $.fx.off = true;
      strictEqual($(DISTANCE).text(), "1 mi");
    });
  });
}());