define([
    'jquery',
    'control',
    'model',
    'map',
    'route'
  ],

  function (
    $,
    Controls,
    Model,
    Map,
    Route) {

    "use strict";
    var gmap;
    var positions = google.maps.ControlPosition;
    var model;
    var route;
    var OTGM = window.OTGM || {};
    var oldConfirm;
    var confirmResponse;

    module("Control Tests", {
      setup: function () {
        gmap = Map.attachTo("#qunit-fixture");
        model = new Model();
        OTGM.standalone = false;
        route = new Route(gmap, model);
        oldConfirm = window.confirm;
        confirmResponse = true;
        window.confirm = function () { return confirmResponse; };
        window.ga = sinon.stub();
      },
      teardown: function () {
        window.confirm = oldConfirm;
      }
    });

    test("Abstract control adding code", function () {
      Controls.build(gmap, model, {}, "<div id='controls'>" +
        "<div data-pos='TOP_LEFT'>" +
        '<div data-idx="-1"></div>' +
        "</div>" +
        "<div data-pos='TOP_RIGHT'>" +
        "<div data-dropdown='abc'></div>" +
        "<div id='abc' style='height: 100px;'></div>" +
        "</div>" +
        "</div>");
      strictEqual(1, gmap.controls[positions.TOP_LEFT].length);
      strictEqual(-1, gmap.controls[positions.TOP_LEFT][0].index);
      strictEqual(2, gmap.controls[positions.TOP_RIGHT].length);
      strictEqual('undefined', typeof gmap.controls[positions.TOP_RIGHT][0].index);
    });

    test("warning displayed when saved from previous", function () {
      model.setLegal({
        warnings: ["warning a"],
        copyrights: ["copyright"]
      });
      var controls = "<div id='controls'>" +
        "<div data-pos='TOP_LEFT'>" +
        "<div class='warning'></div>" +
        "</div>" +
        "</div>";
      var $controls = Controls.build(gmap, model, {}, controls);
      var $warning = $controls.find(".warning");
      strictEqual($warning.css("display"), "block");
    });

    test("shake and confirm yes", function () {
      var removed = false;
      Controls.build(gmap, model, route, "");
      route.lastPoint = {remove: function () {removed = true;}};
      confirmResponse = true;
      $(window).trigger('shake');
      ok(removed, "shake and confirm should clear route");
      deepEqual(window.ga.args, [['send', 'event', 'Shake', "yes"]]);
    });

    test("shake and confirm no", function () {
      var removed = false;
      Controls.build(gmap, model, route, "");
      route.lastPoint = {remove: function () {removed = true;}};
      confirmResponse = false;
      $(window).trigger('shake');
      ok(!removed, "shake and deny should not clear route");
      deepEqual(window.ga.args, [['send', 'event', 'Shake', "no"]]);
    });
  }
);