define([
    'jquery',
    'Modernizr',
    'fastclick'
  ],

  function (
    $,
    Modernizr) {

    "use strict";

    var $div, $parent, clicked, clock, elem;
    var touch = function (name, x, y) {
      var evt = document.createEvent('MouseEvents');
      evt.touches = [{
        clientX: x,
        clientY: y
      }];
      evt.initEvent('touch' + name, true, true);
      $div[0].dispatchEvent(evt);
    };
    var click = function (x, y) {
      var evt;
      if (document.createEvent) {
        evt = document.createEvent('MouseEvents');
        evt.initMouseEvent("click", true, true, window, 0, 0, x, y, 0, false, false, false, false, 0, null);
        $div[0].dispatchEvent(evt);
      } else {
        var options = {};
        options.clientX = x;
        options.clientY = x;
        evt = document.createEventObject();
        var oEvent = $.extend(evt, options);
        $div[0].fireEvent('onclick', oEvent);
      }
    };

    module("Fastclick tests", {
      setup: function () {
        clock = sinon.useFakeTimers();
        Modernizr.usefastclick = true;
        $parent = $("<div>")
          .addClass("usefastclick")
          .appendTo("#qunit-fixture");
        $div = $("<div>")
          .appendTo($parent);
        $div.fastClick(function () {
          clicked++;
          elem = this;
        });
        clicked = 0;
        elem = null;
      },
      teardown: function () {
        clock.tick(10000);
        clock.restore();
      }
    });

    test("regular click event", function () {
      $div.click();
      ok(clicked);
      strictEqual($div[0], elem);
    });

    if (document.addEventListener) {

      test("touch click event", function () {
        touch('start', 0, 0);
        ok(!clicked);
        touch('end', 0, 0);
        strictEqual(clicked, 1);
      });

      test("fastclick-done fired", function () {
        expect(1);
        $("#qunit-fixture").on('fastclick-done', function () {
          ok(true, "was clicked");
        });
        touch('start', 0, 0);
        touch('end', 0, 0);
      });

      test("2nd click blocked", function () {
        touch('start', 0, 0);
        touch('end', 0, 0);
        click(0, 0);
        strictEqual(clicked, 1);
      });

      test("touch move cancels", function () {
        touch('start', 0, 0);
        touch('move', 100, 100);
        touch('end', 0, 0);
        strictEqual(clicked, 0);
      });

      test("touch start propagates", function () {
        var propagated = false;
        $(document).on('touchstart', function () {
          propagated = true;
        });
        touch('start', 0, 0);
        ok(propagated, 'touchstart should propagate');
      });

      test("class added while touching", function () {
        ok(!$div.hasClass("fastclicking"));
        touch('start', 0, 0);
        clock.tick(100);
        ok($div.hasClass("fastclicking"));
        touch('end', 0, 0);
        ok(!$div.hasClass("fastclicking"));
      });
      test("class not added if touch ended", function () {
        touch('start', 0, 0);
        touch('end', 0, 0);
        clock.tick(100);
        ok(!$div.hasClass("fastclicking"));
      });
      test("touch click not invoked if don't have usefastclick class", function () {
        var thisClicked = 0;
        $parent.removeClass("usefastclick").addClass("no-usefastclick");
        Modernizr.usefastclick = false;
        $div.fastClick(function () {
          thisClicked++;
        });
        touch('start', 0, 0);
        touch('end', 0, 0);
        ok(!thisClicked);
      });
    }
  }
);