/*global sinon*/
define(['gmapsingleclick'],

  function (setupSingleClick) {

    "use strict";

    var events = google.maps.event;

    //////////////// single click event
    var clock;
    var mvcObj;
    module("Single click events", {
      setup: function () {
        clock = sinon.useFakeTimers();
        mvcObj = new google.maps.Marker({});
        setupSingleClick(mvcObj);
      },
      teardown: function () {
        clock.tick(1000);
        clock.restore();
      }
    });

    test("single click event", function () {
      var callback = sinon.spy();
      events.addListener(mvcObj, 'singleclick', callback);

      // trigger click
      events.trigger(mvcObj, 'click', "arg");
      deepEqual(callback.args, []);

      // wait 250 ms, should have invoked callback
      clock.tick(250);
      deepEqual(callback.args, [["arg"]]);
    });

    test("two single click events, but no timeout", function () {
      var callback = sinon.spy();
      events.addListener(mvcObj, 'singleclick', callback);

      // trigger click
      events.trigger(mvcObj, 'click', "arg 1");

      clock.tick(125);
      events.trigger(mvcObj, 'click', "arg 2");
      clock.tick(124);
      deepEqual(callback.args, []);

      clock.tick(1);
      deepEqual(callback.args, [["arg 1"]]);
      clock.tick(125);
      deepEqual(callback.args, [["arg 1"], ["arg 2"]]);
    });

    test("single click, canceled by a double click", function () {
      var callback = sinon.spy();
      events.addListener(mvcObj, 'singleclick', callback);

      // trigger click
      events.trigger(mvcObj, 'click', "click 1");

      // wait 249 ms, should not have invoked callback yet, invoke a double
      // click and it should cancel the click event turning into a single click
      clock.tick(249);
      events.trigger(mvcObj, 'click', "click 2");
      events.trigger(mvcObj, 'dblclick', "other");
      deepEqual(callback.args, []);

      clock.tick(1000);
      deepEqual(callback.args, []);
    });
  }
);