define([
    'model',
    'store'
  ],

  function (
    Model,
    Store) {

    "use strict";

    var TravelMode = google.maps.TravelMode;
    var DEFAULT_TRAVEL_MODE = TravelMode.WALKING;

    module("Model tests", {
      setup: function () {
        Store.clear();
      }
    });


    test("travel mode default", function () {
      var model = new Model();
      strictEqual(model.getTravelMode(), DEFAULT_TRAVEL_MODE);
    });
    test("travel invalid travel mode ignored", function () {
      Store.set('travelMode', "garbage");
      var model = new Model();
      strictEqual(model.getTravelMode(), DEFAULT_TRAVEL_MODE);
    });
    test("biking travel mode persisted", function () {
      new Model().setTravelMode(TravelMode.BICYCLING);
      strictEqual(new Model().getTravelMode(), TravelMode.BICYCLING);
    });
    test("walking travel mode persisted", function () {
      new Model().setTravelMode(TravelMode.BICYCLING);
      new Model().setTravelMode(TravelMode.WALKING);
      strictEqual(new Model().getTravelMode(), TravelMode.WALKING);
    });
    test("driving travel mode persisted", function () {
      new Model().setTravelMode(TravelMode.DRIVING);
      strictEqual(new Model().getTravelMode(), TravelMode.DRIVING);
    });
    test("direct travel mode persisted", function () {
      new Model().setTravelMode(null);
      strictEqual(new Model().getTravelMode(), null);
    });

    test("default unit", function () {
      strictEqual(new Model().getUnit(), "mi");
    });
    test("km persisted", function () {
      new Model().toggleUnit();
      strictEqual(new Model().getUnit(), "km");
    });
    test("garbage unit ignored", function () {
      Store.set("unit", "jlkj");
      strictEqual(new Model().getUnit(), "mi");
    });
    test("mi persisted", function () {
      var model = new Model();
      model.toggleUnit();
      model.toggleUnit();
      strictEqual(new Model().getUnit(), "mi");
    });


    test("default bike layer", function () {
      strictEqual(new Model().getBikeLayer(), false);
    });
    test("bike layer persisted", function () {
      new Model().setBikeLayer(true);
      strictEqual(new Model().getBikeLayer(), true);
    });
    test("no bike layer persisted", function () {
      new Model().setBikeLayer(true);
      new Model().setBikeLayer(false);
      strictEqual(new Model().getBikeLayer(), false);
    });
    test("garbage bike layer ignored", function () {
      Store.set("bikeLayer", "jlkj");
      strictEqual(new Model().getBikeLayer(), false);
    });

    // ensure that data stored in client browsers remains compatible
    test("snapshot stored data behavior", function () {
      Store.set("bikeLayer", "true");
      Store.set("travelMode", "WALKING");
      Store.set("unit", "km");
      var model = new Model();
      strictEqual(model.getBikeLayer(), true);
      strictEqual(model.getTravelMode(), TravelMode.WALKING);
      strictEqual(model.getUnit(), "km");
    });
  }
);