define([
    'jquery',
    'rdp'
  ],

  function (
    $,
    RDP) {

    "use strict";
    var id = function (n) { return n; };
    var maxlen = function (max) {
      return function (a) {
        return a.length / (max + 1);
      };
    };
    var enumerate = function (n) {
      var result = [];
      for (var i = 0; i < n; i++) {
        result.push(i);
      }
      return result;
    };
    var ll = function (lat, lng) {
      return new google.maps.LatLng(lat, lng);
    };
    var pointsEqual = function (a, b, msg) {
      var pointToString = function (point) {
        return point.toString();
      };
      var aStr = $.map(a, pointToString);
      var bStr = $.map(b, pointToString);
      deepEqual(aStr, bStr, msg);
    };
    var clock;
    module("Ramer-Douglas-Peucker algorithm tests");

    test("Line distance - no length", function () {
      var l = new RDP.Line(ll(0, 0), ll(0, 0));
      var run = function (point, expectedDist) {
        var latlng = ll(point[0], point[1]);
        strictEqual(l.distanceTo(latlng), expectedDist,
          latlng + " should be " + expectedDist);
      };
      run([0, 1], 1);
      run([0, 2], 2);
      run([0, -2], 2);
      run([1, 0], 1);
      run([0, 0], 0);
    });

    test("Line distance - horizontal", function () {
      var l = new RDP.Line(ll(0, 0), ll(5, 0));
      var run = function (point, expectedDist) {
        var latlng = ll(point[0], point[1]);
        strictEqual(l.distanceTo(latlng), expectedDist,
          latlng + " should be " + expectedDist);
      };
      run([1, 1], 1);
      run([0, 2], 2);
      run([1, -2], 2);
      run([1, 0], 0);
      run([0, -1], 1);

      run([-0.5, 0], 0.5);
      run([5.5, 0], 0.5);
    });

    test("Line distance - vertical", function () {
      var l = new RDP.Line(ll(0, 0), ll(0, 5));
      var run = function (point, expectedDist) {
        var latlng = ll(point[0], point[1]);
        strictEqual(l.distanceTo(latlng), expectedDist,
          latlng + " should be " + expectedDist);
      };
      run([1, 1], 1);
      run([2, 0], 2);
      run([-2, 1], 2);
      run([0, 1], 0);
      run([-1, 0], 1);

      run([0, -0.5], 0.5);
      run([0, 5.5], 0.5);
    });

    test("maximize empty", function () {
      var result = RDP.maximize({
        length: 0,
        getFirstN: enumerate
      }, 0, maxlen(0), id);
      deepEqual(result.encoded, []);

      result = RDP.maximize({
        length: 0,
        getFirstN: enumerate
      }, -1, maxlen(0), id);
      deepEqual(result.encoded, []);

      result = RDP.maximize({
        length: 0,
        getFirstN: enumerate
      }, 1, maxlen(0), id);
      deepEqual(result.encoded, []);
    });

    test("maximize length 1", function () {
      var result = RDP.maximize({
        length: 1,
        getFirstN: enumerate
      }, 0, maxlen(3), id);
      deepEqual(result.encoded, [0]);
    });

    test("maximize length 3", function () {
      var result = RDP.maximize({
        length: 3,
        getFirstN: enumerate
      }, 0, maxlen(3), id);
      deepEqual(result.encoded, [0, 1, 2]);
    });

    test("maximize length 3 - start at 10", function () {
      var result = RDP.maximize({
        length: 3,
        getFirstN: enumerate
      }, 10, maxlen(3), id);
      deepEqual(result.encoded, [0, 1, 2]);
    });

    test("maximize length 6 - can only take 3", function () {
      var result = RDP.maximize({
        length: 6,
        getFirstN: enumerate
      }, 1, maxlen(3), id);
      deepEqual(result.encoded, [0, 1, 2]);
    });

    asyncTest("Use RDP to characterize empty list", function () {
      var points = [];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 0);
        pointsEqual(data.getFirstN(0), []);
        pointsEqual(data.getFirstN(1), []);
        start();
      });
    });

    asyncTest("Use RDP to characterize length 1 list", function () {
      var points = [ll(0, 0)];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 1);
        pointsEqual(data.getFirstN(0), [ll(0, 0)]);
        pointsEqual(data.getFirstN(1), [ll(0, 0)]);
        start();
      });
    });

    asyncTest("Use RDP to characterize length 2 list", function () {
      var points = [ll(0, 0), ll(0, 1)];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 2);
        pointsEqual(data.getFirstN(0), [ll(0, 0), ll(0, 1)]);
        pointsEqual(data.getFirstN(1), [ll(0, 0), ll(0, 1)]);
        pointsEqual(data.getFirstN(2), [ll(0, 0), ll(0, 1)]);
        start();
      });
    });

    asyncTest("Use RDP to characterize length 3 list", function () {
      var points = [ll(0, 0), ll(0.1, 1), ll(0, 2)];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 3);
        pointsEqual(data.getFirstN(2), [ll(0, 0), ll(0, 2)]);
        pointsEqual(data.getFirstN(3), [ll(0, 0), ll(0.1, 1), ll(0, 2)]);
        start();
      });
    });

    asyncTest("Use RDP to characterize length 3 list", function () {
      var points = [ll(0, 0), ll(0.1, 1), ll(0, 2)];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 3);
        pointsEqual(data.getFirstN(2), [ll(0, 0), ll(0, 2)]);
        pointsEqual(data.getFirstN(3), [ll(0, 0), ll(0.1, 1), ll(0, 2)]);
        start();
      });
    });

    asyncTest("Use RDP to characterize length 4 list", function () {
      var points = [ll(0, 0), ll(2, 1), ll(1, 2), ll(0, 3)];
      RDP.characterize(points, function (data) {
        strictEqual(data.length, 4);
        pointsEqual(data.getFirstN(2), [ll(0, 0), ll(0, 3)]);
        pointsEqual(data.getFirstN(3), [ll(0, 0), ll(2, 1), ll(0, 3)]);
        pointsEqual(data.getFirstN(4), [ll(0, 0), ll(2, 1), ll(1, 2), ll(0, 3)]);
        start();
      });
    });

    var data = [ll(0, 0), ll(2, 1), ll(1, 2), ll(0, 3)];
    module("RDP Cancel tests", {
      setup: function () {
        clock = sinon.useFakeTimers();
      },
      teardown: function () {
        clock.tick(1000);
        clock.restore();
      }
    });

    test("Making RDP request cancels previous", function () {
      var spy = sinon.spy();
      var spy2 = sinon.spy();
      RDP.characterize(data, spy);
      RDP.characterize(data, spy2);
      clock.tick(1000);
      deepEqual(spy.args.length, 0, "calls to 1st");
      deepEqual(spy2.args.length, 1, "calls to 2nd");
    });

    test("Cancel RDP", function () {
      var spy = sinon.spy();
      RDP.characterize(data, spy);
      RDP.cancel();
      clock.tick(1000);
      deepEqual(spy.args.length, 0, "calls to 1st");
    });
  }
);