define([
    'sample'
  ],

  function (sample) {

    "use strict";

    module("Sample tests");

    var ll = function (lat, lng) {
      return new google.maps.LatLng(lat, lng);
    };

    var unwrap = function (point) {
      return [point.lat(), point.lng()].join(", ");
    };

    var samplesShouldBe = function (actual, expected) {
      actual = actual.map(unwrap);
      expected = expected.map(unwrap);
      deepEqual(actual, expected);
    };

    test("sample empty path @0", function () {
      var path = [];
      var sampled = sample({
        path: path,
        numPoints: 0
      });
      samplesShouldBe(sampled, []);
    });

    test("sample empty path @1", function () {
      var path = [];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, []);
    });

    test("sample empty path @2", function () {
      var path = [];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, []);
    });

    test("sample length-1 path @0", function () {
      var path = [ll(0, 0)];
      var sampled = sample({
        path: path,
        numPoints: 0
      });
      samplesShouldBe(sampled, [ll(0, 0)]);
    });

    test("sample length-1 path @1", function () {
      var path = [ll(0, 0)];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, [ll(0, 0)]);
    });

    test("sample length-1 path @2", function () {
      var path = [ll(0, 0)];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, [ll(0, 0)]);
    });

    test("sample length-2 path @0", function () {
      var path = [ll(0, 0), ll(0, 1)];
      var sampled = sample({
        path: path,
        numPoints: 0
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 1)]);
    });

    test("sample length-2 path @1", function () {
      var path = [ll(0, 0), ll(0, 1)];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 1)]);
    });

    test("sample length-2 path @2", function () {
      var path = [ll(0, 0), ll(0, 1)];
      var sampled = sample({
        path: path,
        numPoints: 1
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 1)]);
    });

    test("sample length-3 path @0", function () {
      var path = [ll(0, 0), ll(0, 1), ll(0, 2)];
      var sampled = sample({
        path: path,
        numPoints: 0
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 2)]);
    });

    test("sample length-3 path @3", function () {
      var path = [ll(0, 0), ll(0, 1), ll(0, 2)];
      var sampled = sample({
        path: path,
        numPoints: 3
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 1), ll(0, 2)]);
    });

    test("sample length-2 path @3", function () {
      var path = [ll(0, 0), ll(0, 1)];
      var sampled = sample({
        path: path,
        numPoints: 3
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 0.5), ll(0, 1)]);
    });

    test("sample length-2 path @4", function () {
      var path = [ll(0, 0), ll(0, 1)];
      var sampled = sample({
        path: path,
        numPoints: 4
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 1/3), ll(0, 2/3), ll(0, 1)]);
    });

    test("sample length-4 path @2", function () {
      var path = [ll(0, 0), ll(0, 1), ll(0, 2), ll(0, 3)];
      var sampled = sample({
        path: path,
        numPoints: 2
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 3)]);
    });

    test("sample length-5 path @3", function () {
      var path = [ll(0, 0), ll(0, 1), ll(0, 2), ll(0, 3), ll(0, 4)];
      var sampled = sample({
        path: path,
        numPoints: 3
      });
      samplesShouldBe(sampled, [ll(0, 0), ll(0, 2), ll(0, 4)]);
    });
  }
);