define([
    'segment',
    'point',
    'map',
    'directions',
    'util'
  ],

  function (
    Segment,
    Point,
    Map,
    Directions,
    Util) {

    "use strict";
    var LatLng = google.maps.LatLng;
    var TravelMode = google.maps.TravelMode;
    var map;
    var route = {
      update: function () {},
      addPoint: function () {}
    };
    var failOnGetRequests = function () {
      Directions.get = function () {
        throw new Error("Expected no get requests");
      };
    };
    var allowGetRequests = function () {
      Directions.get = function () {};
    };
    var extractPolylineCoords = function (poly) {
      var path = poly.getPath();
      var result = [];
      path.forEach(function (point) {
        result.push([
          Util.round(point.lat(), 3),
          Util.round(point.lng(), 3)
        ]);
      });
      return result;
    };
    var oldGet;

    module("Segment tests", {
      setup: function () {
        oldGet = Directions.get;
        map = Map.attachTo("#qunit-fixture");
        allowGetRequests();
      },
      teardown: function () {
        Directions.get = oldGet;
      }
    });

    test("test serialize/deserialize happy path direct", function () {
      var start = new Point(map, new LatLng(1, 2), route);
      var end = new Point(map, new LatLng(3, 4), route);
      var segment = new Segment(map, null, start, end, route);
      var serialized = segment.serialize();
      var deserialized = new Segment(map, null, start, end, route, serialized);
      deepEqual(deserialized.polyline.getPath(),
        segment.polyline.getPath(),
        "deserialized path");
    });

    test("test distance saved", function () {
      var start = new Point(map, new LatLng(1, 2), route);
      var end = new Point(map, new LatLng(3, 4), route);
      var segment = new Segment(map, null, start, end, route);
      segment.distance = 100;
      var serialized = segment.serialize();
      var deserialized = new Segment(map, null, start, end, route, serialized);
      strictEqual(deserialized.distance, 100, "saved distance");
    });


    test("test routed polyline saved", function () {
      var start = new Point(map, new LatLng(1, 2), route);
      var end = new Point(map, new LatLng(3, 4), route);
      var segment = new Segment(map, TravelMode.WALKING, start, end, route);
      segment.handleDirectionResponse(
        ["warning", "warning2"],
        "copyright",
        {
          distance: 10,
          points: [new LatLng(1, 2), new LatLng(0, 0), new LatLng(3, 4)]
        },
        {
          distance: 10,
          points: [new LatLng(3, 4), new LatLng(0, 0), new LatLng(1, 2)]
        },
        {}
      );
      var serialized = segment.serialize();
      failOnGetRequests();
      var deserialized = new Segment(map, TravelMode.WALKING, start, end, route, serialized);

      strictEqual(deserialized.distance, 10,
        "deserialized distance");
      strictEqual(deserialized.copyright, "copyright",
        "deserialized copyright");
      deepEqual(deserialized.warnings, ["warning", "warning2"],
        "deserialized warnings");
      deepEqual(
        extractPolylineCoords(deserialized.polyline),
        extractPolylineCoords(segment.polyline),
        "deserialized polyline");
      strictEqual(deserialized.travelMode, TravelMode.WALKING,
        "deserialized travel mode");
    });

    // ensure that single click event listener set up
    test("single click is set up", function () {
      var start = new Point(map, new LatLng(1, 2), route);
      var end = new Point(map, new LatLng(3, 4), route);
      var segment = new Segment(map, null, start, end, route);
      expect(1);
      var clock = sinon.useFakeTimers();
      try {
        google.maps.event.addListener(segment.mouseoverpoly, 'singleclick', function () {
          ok(true, "slick click called");
        });
        google.maps.event.trigger(segment.mouseoverpoly, 'click', {latLng: new google.maps.LatLng(1, 1)});
        clock.tick(251);
      } finally {
        clock.restore();
      }
    });
  }
);