// tests for shake detection
define([
    'jquery',
    'shake'
  ],

  function (
    $,
    shake) {

    "use strict";

    var clock;
    var shook;
    var JUST_OVER_THRESHOLD = 7.01;
    var JUST_UNDER_THRESHOLD = 6.9;
    var accel = function (dir) {
      shake.handleDeviceMotion({
        originalEvent: {
          acceleration: dir
        }
      });
    };

    module("Shake tests", {
      setup: function () {
        $(window).off('shake');
        $(window).off('devicemotion');
        clock = sinon.useFakeTimers();
        shook = 0;
        $(window).on('shake', function () {
          shook++;
        });
        shake.reset();
      },
      teardown: function () {
        clock.tick(100000);
        clock.restore();
      }
    });

    test("1 turns doesn't shake", function () {
      accel({x: -JUST_OVER_THRESHOLD});
      ok(!shook);
    });

    test("2 turns doesn't shake", function () {
      accel({x: -JUST_OVER_THRESHOLD});
      accel({x: JUST_OVER_THRESHOLD});
      ok(!shook);
    });

    test("3 turns does shake", function () {
      accel({x: -JUST_OVER_THRESHOLD});
      accel({x: JUST_OVER_THRESHOLD});
      clock.tick(999);
      accel({x: -JUST_OVER_THRESHOLD});
      ok(shook);
    });

    test("3 turns just under threshold doesn't shake", function () {
      accel({x: -JUST_UNDER_THRESHOLD});
      accel({x: JUST_UNDER_THRESHOLD});
      accel({x: -JUST_UNDER_THRESHOLD});
      ok(!shook);
    });

    test("3 turns in >1s doesn't shake", function () {
      accel({x: -JUST_OVER_THRESHOLD});
      accel({x: JUST_OVER_THRESHOLD});
      clock.tick(1001);
      accel({x: -JUST_OVER_THRESHOLD});
      ok(!shook);
    });

    test("3 turns in >1s doesn't shake (2)", function () {
      accel({x: -JUST_OVER_THRESHOLD});
      clock.tick(1001);
      accel({x: JUST_OVER_THRESHOLD});
      accel({x: -JUST_OVER_THRESHOLD});
      ok(!shook);
    });

    test("3 turns does shake (y)", function () {
      accel({y: -JUST_OVER_THRESHOLD});
      accel({y: JUST_OVER_THRESHOLD});
      accel({y: -JUST_OVER_THRESHOLD});
      ok(shook);
    });

    test("3 turns does shake (z)", function () {
      accel({z: -JUST_OVER_THRESHOLD});
      accel({z: JUST_OVER_THRESHOLD});
      accel({z: -JUST_OVER_THRESHOLD});
      ok(shook);
    });

    test("3 turns in x/y/z shakes only once", function () {
      accel({x: -JUST_OVER_THRESHOLD, y: -JUST_OVER_THRESHOLD, z: -JUST_OVER_THRESHOLD});
      accel({x: JUST_OVER_THRESHOLD, y: JUST_OVER_THRESHOLD, z: JUST_OVER_THRESHOLD});
      accel({x: -JUST_OVER_THRESHOLD, y: -JUST_OVER_THRESHOLD, z: -JUST_OVER_THRESHOLD});
      strictEqual(shook, 1);
    });
  }
);