define([
    'util'
  ],

  function (util) {

    "use strict";
    var LatLng = google.maps.LatLng;
    module('Utility tests');

    test("Test round()", function () {
      strictEqual(1, util.round(1.11));
      strictEqual(1.1, util.round(1.11, 1));
      strictEqual(1, util.round(1.11, 0));
      strictEqual(10, util.round(11, -1));
      strictEqual(2, util.round(1.5, 0));
      strictEqual(1, util.round(1.49, 0));
    });

    test("Test numersAreClose()", function () {
      ok(util.numbersAreClose(0, 0));
      ok(!util.numbersAreClose(0, 1));
      ok(util.numbersAreClose(0, 1e-5));
      ok(util.numbersAreClose(1e-5, 0));
      ok(!util.numbersAreClose(0, 2e-5));
      ok(!util.numbersAreClose(2e-5, 0));
    });

    test("Test pointsAreClose()", function () {
      var run = function (a, b) {
        return util.pointsAreClose(new LatLng(a[0], a[1]), new LatLng(b[0], b[1]));
      };
      ok(run([0, 0], [0, 0]));
      ok(run([0, 0], [1e-5, 1e-5]));
      ok(run([1e-5, 1e-5], [0, 0]));
      ok(run([0, 0], [-1e-5, -1e-5]));
      ok(run([-1e-5, -1e-5], [0, 0]));
      ok(!run([-1e-5, 0], [1e-5, 0]));
      ok(!run([0, 1e-5], [0, -1e-5]));
    });

    test("Test invert()", function () {
      deepEqual(util.invert({}), {});
      deepEqual(util.invert({
        a: 1
      }), {
        1: "a"
      });
      deepEqual(util.invert({
        a: 1,
        b: 2
      }), {
        1: "a",
        2: "b"
      });
      raises(function () {
        util.invert({a: "c", b: "c"});
      }, function (e) {
        return e.message === "Both 'a' and 'b' map to 'c'";
      });
    });
  }
);